"""
This file is part of the 'Elements' Project
Elements is a 2D Physics API for Python (supporting Box2D2)

Copyright (C) 2008, The Elements Team, <elements@linuxuser.at>

Home:  http://elements.linuxuser.at
IRC:   #elements on irc.freenode.org

Code:  http://www.assembla.com/wiki/show/elements
       svn co http://svn2.assembla.com/svn/elements                     

License:  GPLv3 | See LICENSE for the full text
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.              
"""
from locals import *
from elements import box2d

class CallbackHandler:
    # List of contact callbacks and shapes to start them - sorted by type for quicker access  
    # Callbacks are saved as callbacks[callback_type][[function, parameters], ...]
    callbacks = {}   
    
    def __init__(self, parent):
        self.parent = parent
        
        # init callback dict to avoid those slow try 
        # (especially for self.get, as it is called *often*)
        for i in xrange(10):
            self.callbacks[i] = []
         
    def add(self, callback_type, callback_handler, *args):
            """ Users can add callbacks for certain (or all) collisions
               
               Parameters:
                 callback_type ......... CALLBACK_CONTACT (nothing else for now)
                 callback_handler ...... a callback function
                 args (optional) ....... a list of parameters which can be used with callbacks.get
               
               Return:
                 callback_id ... used to remove a callback later (int)
            """
            # Create contact listener if required
            if callback_type in [CALLBACK_CONTACT_ADD, CALLBACK_CONTACT_PERSIST, CALLBACK_CONTACT_REMOVE]:
                if self.parent.listener == None:
                    self.parent.listener = kContactListener(self.get)
                    self.parent.world.SetContactListener( self.parent.listener )
                    print "* ContactListener added"
            
            # Get callback dict for this callback_type
            c = self.callbacks[callback_type]
                        
            # Append to the Callback Dictionary
            c.append([callback_handler, args])
            self.callbacks[callback_type] = c
            
            # Return Callback ID
            # ID = callback_type.callback_index (1...n)
            return "%i.%i" % (callback_type, len(c))
        
    def get(self, callback_type):
        return self.callbacks[callback_type]
            
    def start(self, callback_type, *args):
        callbacks = self.get(callback_type)
        for c in callbacks:
            callback, params = c
            callback()

class kContactListener(box2d.b2ContactListener):
    def __init__(self, get_callbacks): 
        # Init the Box2D b2ContactListener
        box2d.b2ContactListener.__init__(self)

        # Function to get the current callbacks
        self.get_callbacks = get_callbacks
        
    def check_contact(self, contact_type, point):
        # Checks if a callback should be started with this contact point
        contacts = self.get_callbacks(contact_type)

        # Step through all callbacks for this type (eg ADD, PERSIST, REMOVE)
        for c in contacts:
            callback, bodylist = c
            if len(bodylist) == 0:
                # Without bodylist it's a universal callback (for all bodies) 
                callback(point)
                
            else:
                # This is a callback with specified bodies
                # See if this contact involves one of the specified
                b1 = str(point.shape1.GetBody())
                b2 = str(point.shape2.GetBody())
                for s in bodylist:
                    s = str(s)
                    if b1 == s or b2 == s: 
                        # Yes, that's the one :)
                        callback(point)
         
    def Add(self, point):
        """Called when a contact point is created"""
        self.check_contact(CALLBACK_CONTACT_ADD, point)
        
    def Persist(self, point):
        """Called when a contact point persists for more than a time step"""
        self.check_contact(CALLBACK_CONTACT_PERSIST, point)
                        
    def Remove(self, point):
        """Called when a contact point is removed"""
        self.check_contact(CALLBACK_CONTACT_REMOVE, point)
        



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import random
import pilas


class Habilidad:

    def __init__(self, receptor):
        self.receptor = receptor

    def actualizar(self):
        pass

    def eliminar(self):
        pass

class RebotaComoPelota(Habilidad):

    def __init__(self, receptor):
        Habilidad.__init__(self, receptor)
        error = random.randint(-10, 10) / 10.0
        self.figura = pilas.fisica.fisica.crear_figura_circulo(receptor.x + error, 
                                                               receptor.y + error, 
                                                               receptor.radio_de_colision,
                                                               masa=10,
                                                               elasticidad=0.75)
        self.ultimo_x = receptor.x
        self.ultimo_y = receptor.y

    def actualizar(self):
        # Mueve el objeto siempre y cuando no parezca que algo
        # no fisico (es decir de pymunk) lo ha afectado.
        if self.ultimo_x == self.receptor.x and self.ultimo_y == self.receptor.y:
            self.receptor.x = self.figura.body.position.x
            self.receptor.y = self.figura.body.position.y
            self.receptor.rotacion = self.figura.body.angle * 500

            self.ultimo_x = self.receptor.x
            self.ultimo_y = self.receptor.y
        else:
            # Si ha detectado que el usuario u alguna sentencia
            # de código alteró la posicion x e y entonces le
            # avisa a pymunk que aplique ese cambio de posicion.
            self.ultimo_x = self.receptor.x
            self.ultimo_y = self.receptor.y
            self.figura.body.position.x = self.ultimo_x
            self.figura.body.position.y = self.ultimo_y

    def eliminar(self):
        pilas.fisica.fisica.eliminar(self.figura)


class RebotaComoCaja(Habilidad):

    def __init__(self, receptor):
        Habilidad.__init__(self, receptor)
        error = random.randint(-10, 10) / 10.0
        self.figura = pilas.fisica.fisica.crear_figura_cuadrado(receptor.x + error,
                                                               receptor.y + error, 
                                                               receptor.radio_de_colision,
                                                               masa=10,
                                                               elasticidad=0.30,
                                                               friccion=10)
        self.ultimo_x = receptor.x
        self.ultimo_y = receptor.y
        
    def actualizar(self):
        # Mueve el objeto siempre y cuando no parezca que algo
        # no fisico (es decir de pymunk) lo ha afectado.
        if self.ultimo_x == self.receptor.x and self.ultimo_y == self.receptor.y:
            self.receptor.x = self.figura.body.position.x
            self.receptor.y = self.figura.body.position.y
            self.receptor.rotacion = self.figura.body.angle * 58
        else:
            # Si ha detectado que el usuario u alguna sentencia
            # de código alteró la posicion x e y entonces le
            # avisa a pymunk que aplique ese cambio de posicion.
            self.ultimo_x = self.receptor.x
            self.ultimo_y = self.receptor.y
            self.figura.body.position.x = self.ultimo_x
            self.figura.body.position.y = self.ultimo_y

    def eliminar(self):
        pilas.fisica.fisica.eliminar(self.figura)

class ColisionableComoPelota(RebotaComoPelota):

    def __init__(self, receptor):
        RebotaComoPelota.__init__(self, receptor)
        
    def actualizar(self):
        self.figura.body.position.x = self.receptor.x
        self.figura.body.position.y = self.receptor.y

    def eliminar(self):
        pilas.fisica.fisica.eliminar(self.figura)

class SeguirAlMouse(Habilidad):
    "Hace que un actor siga la posición del mouse en todo momento."

    def __init__(self, receptor):
        Habilidad.__init__(self, receptor)
        pilas.eventos.mueve_mouse.connect(self.mover)

    def mover(self, evento):
        self.receptor.x = evento.x
        self.receptor.y = evento.y

class AumentarConRueda(Habilidad):
    "Permite cambiar el tamaño de un actor usando la ruedita scroll del mouse."

    def __init__(self, receptor):
        Habilidad.__init__(self, receptor)
        pilas.eventos.mueve_rueda.connect(self.cambiar_de_escala)

    def cambiar_de_escala(self, evento):
        self.receptor.escala += (evento.delta / 4.0)


class SeguirClicks(Habilidad):
    "Hace que el actor se coloque la posición del cursor cuando se hace click."

    def __init__(self, receptor):
        Habilidad.__init__(self, receptor)
        pilas.eventos.click_de_mouse.connect(self.moverse_a_este_punto)

    def moverse_a_este_punto(self, evento):
        self.receptor.x = pilas.interpolar(evento.x, duracion=0.5)
        self.receptor.y = pilas.interpolar(evento.y, duracion=0.5)


class Arrastrable(Habilidad):
    """Hace que un objeto se pueda arrastrar con el puntero del mouse.

    Cuando comienza a mover al actor se llama al metodo ''comienza_a_arrastrar''
    y cuando termina llama a ''termina_de_arrastrar''. Estos nombres
    de metodos se llaman para que puedas personalizar estos eventos, dado
    que puedes usar polimorfismo para redefinir el comportamiento
    de estos dos metodos. Observa un ejemplo de esto en
    el ejemplo ``pilas.ejemplos.Piezas``.
    """

    def __init__(self, receptor):
        Habilidad.__init__(self, receptor)
        pilas.eventos.click_de_mouse.connect(self.try_to_drag)

    def try_to_drag(self, evento):
        "Intenta mover el objeto con el mouse cuando se pulsa sobre el."

        if self.receptor.colisiona_con_un_punto(evento.x, evento.y):
            pilas.eventos.termina_click.connect(self.drag_end)
            pilas.eventos.mueve_mouse.connect(self.drag, uid='drag')
            self.comienza_a_arrastrar()

    def drag(self, evento):
        "Arrastra el actor a la posicion indicada por el puntero del mouse."
        self.receptor.x += evento.dx
        self.receptor.y += evento.dy

    def drag_end(self, evento):
        "Suelta al actor porque se ha soltado el botón del mouse."
        pilas.eventos.mueve_mouse.disconnect(uid='drag')
        self.termina_de_arrastrar()

    def comienza_a_arrastrar(self):
        pass

    def termina_de_arrastrar(self):
        pass

class MoverseConElTeclado(Habilidad):
    "Hace que un actor cambie de posición con pulsar el teclado."

    def __init__(self, receptor):
        Habilidad.__init__(self, receptor)
        pilas.eventos.actualizar.connect(self.on_key_press)

    def on_key_press(self, evento):
        velocidad = 5
        c = pilas.mundo.control

        if c.izquierda:
            self.receptor.x -= velocidad
        elif c.derecha:
            self.receptor.x += velocidad

        if c.arriba:
            self.receptor.y += velocidad
        elif c.abajo:
            self.receptor.y -= velocidad

class PuedeExplotar(Habilidad):
    "Hace que un actor se pueda hacer explotar invocando al metodo eliminar."

    def __init__(self, receptor):
        Habilidad.__init__(self, receptor)
        receptor.eliminar = self.eliminar_y_explotar

    def eliminar_y_explotar(self):
        explosion = pilas.actores.Explosion()
        explosion.x = self.receptor.x
        explosion.y = self.receptor.y
        explosion.escala = self.receptor.escala * 2
        pilas.baseactor.BaseActor.eliminar(self.receptor)


class SeMantieneEnPantalla(Habilidad):
    """Se asegura de que el actor regrese a la pantalla si sale.

    Si el actor sale por la derecha de la pantalla, entonces regresa
    por la izquiera. Si sale por arriba regresa por abajo y asi..."""

    def actualizar(self):
        # Se asegura de regresar por izquierda y derecha.
        if self.receptor.derecha < -320:
            self.receptor.izquierda = 320
        elif self.receptor.izquierda > 320:
            self.receptor.derecha = -320

        # Se asegura de regresar por arriba y abajo.
        if self.receptor.abajo > 240:
            self.receptor.arriba = -240
        elif self.receptor.arriba < -240:
            self.receptor.abajo = 240


class PisaPlataformas(Habilidad):

    def __init__(self, receptor):
        Habilidad.__init__(self, receptor)
        error = random.randint(-10, 10) / 10.0
        self.figura = pilas.fisica.fisica.crear_figura_cuadrado(receptor.x + error, 
                                                               receptor.y + error, 
                                                               receptor.radio_de_colision,
                                                               masa=10,
                                                               elasticidad=0,
                                                               friccion=0)
        self.ultimo_x = receptor.x
        self.ultimo_y = receptor.y

    def actualizar(self):
        # Mueve el objeto siempre y cuando no parezca que algo
        # no fisico (es decir de pymunk) lo ha afectado.
        self.receptor.x = self.figura.body.position.x
        self.receptor.y = self.figura.body.position.y

    def eliminar(self):
        pilas.fisica.fisica.eliminar(self.figura)



import pilas


class Ayuda(pilas.escenas.Escena):
    "Escena que entrega instrucciones de como jugar."

    def __init__(self):
        pilas.escenas.Escena.__init__(self) 
        pilas.actores.utils.eliminar_a_todos()
        pilas.fondos.Fondo('data/ayuda.png')
        pilas.eventos.pulsa_tecla_escape.conectar(self.cuando_se_presione_escape)

    def cuando_se_presione_escape(self, *k, **kv):
        "Regresa al menu principal"
        import escena_menu
        escena_menu.EscenaMenu()



# -*- encoding: utf-8 -*-
import pilas


pilas.iniciar()

actor = pilas.actores.Actor()
animacion = pilas.imagenes.Grilla("pingu.png", 10)
animacion.asignar(actor)

def avanzar_cuadro(*k, **kv):
    "Avanza un cuadro de animación."
    animacion.avanzar()
    animacion.asignar(actor)

pilas.eventos.click_de_mouse.connect(avanzar_cuadro)
pilas.avisar("Pulse el boton del mouse para avanzar un cuadro.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti 
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar
#
# Boton Programmed by Pablo Garrido

from pilas.actores import Actor
import pilas

class Boton(Actor):
    """Representa un boton que reacciona al ser presionado."""
	
    def __init__(self, x=0, y=0, 
                ruta_normal = 'boton/boton_normal.png',
                ruta_press = 'boton/boton_press.png',
                ruta_over = 'boton/boton_over.png',
                ):

        self.funciones_normal = []
        self.funciones_press = []
        self.funciones_over = []
        Actor.__init__(self, ruta_normal, x=x, y=y)
        self._cargar_imagenes(ruta_normal, ruta_press, ruta_over)

        pilas.eventos.mueve_mouse.conectar(self.detection_move_mouse)
        pilas.eventos.click_de_mouse.conectar(self.detection_click_mouse)
        pilas.eventos.termina_click.conectar(self.detection_end_click_mouse)

    def _cargar_imagenes(self, ruta_normal, ruta_press, ruta_over):
        self.imagen_over = pilas.imagenes.cargar(ruta_over)
        self.imagen_normal = pilas.imagenes.cargar(ruta_normal)
        self.imagen_press = pilas.imagenes.cargar(ruta_press)
    

    #funciones que conectan evento(press, over, normal) a funciones 
    def conectar_normal(self, funcion):
        self.funciones_normal.append(funcion)
    
    def conectar_presionado(self, funcion):
        self.funciones_press.append(funcion)
    
    def conectar_sobre(self, funcion):
        self.funciones_over.append(funcion)    
    
    def desconectar_normal(self, funcion):
        self.funciones_normal.remove(funcion)
    
    def desconectar_presionado(self, funcion):
        self.funciones_press.remove(funcion)
    
    def desconectar_sobre(self, funcion):
        self.funciones_over.remove(funcion) 

    def ejecutar_funciones_normal(self):
        for i in self.funciones_normal:
            i()
    
    def ejecutar_funciones_press(self):
        for i in self.funciones_press:
            i()
    
    def ejecutar_funciones_over(self):
        for i in self.funciones_over:
            i()

    # funciones que cambian la imagen del boton
    def pintar_normal(self):
        self.definir_imagen(self.imagen_normal)

    def pintar_presionado(self):
        self.definir_imagen(self.imagen_press)

    def pintar_sobre(self):
        self.definir_imagen(self.imagen_over) 

    def detection_move_mouse(self, evento):
        if self.colisiona_con_un_punto(evento.x, evento.y):
            self.ejecutar_funciones_over()
        else:
            self.ejecutar_funciones_normal()

    def detection_click_mouse(self, click):
        if self.colisiona_con_un_punto(click.x, click.y):
            self.ejecutar_funciones_press()

    def detection_end_click_mouse(self, end_click):
        pass



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

class Interpolacion:
    """Representa una interpolacion, que pasa por varios puntos clave.

    Las interpolacione se utilizan para realizar movimientos de
    actores en la pantalla. O simplemente para cambiar el
    estado de un actor de un punto a otro, por ejemplo, de 0 a 360
    grados de manera gradual.

    Todo objeto de interpolaciones se puede asignar directamente a
    una propiedad de un actor. Por ejemplo:

        actor.rotation = pilas.interpolations.Lineal(400)

    note que hay un atajo para usar estos objetos, es mejor
    utilizar directamente una sentencias como la que sigue::

        actor.rotation = pilas.interpolate(360)
    """
    pass


class Lineal(Interpolacion):
    "Representa una interpolación lineal."

    def __init__(self, values, duration, delay):
        """Inicializa la interpolación.

        ``values`` tiene que ser una lista con todos los puntos
        por los que se quiere adoptar valores y ``duration`` es la cantidad
        de segundos que deben tomarse para realizar la interpolación.
        """
        self.values = values
        self.duration = duration
        self.delay = delay

    def __neg__(self):
        "Retorna la interpolación inversa a la original."
        new_values = list(self.values)
        new_values.reverse()
        return Lineal(new_values, self.duration, self.delay)

    def apply(self, target, function):
        """Aplica la interpolación a un actor usando un método.

        Esta funcionalidad se utiliza para que toda interpolación
        se pueda acoplar a un actor.

        La idea es contar con la interpolación, un actor y luego
        ponerla en funcionamiento::

            mi_interpolacion.apply(mono, set_rotation)

        de esta forma los dos objetos están y seguirán estando
        desacoplados."""

        import pilas

        # Tiempo que se debe invertir para hacer cada interpolacion
        # individual.
        step = self.duration / float(len(self.values))
        step *= 1000.0

        # En base a la funcion busca el getter que le dara
        # el valor inicial.
        getter = function.replace('set_', 'get_')
        function_to_get_value = getattr(target, getter)
        fist_value = function_to_get_value()

        # Le indica al objeto que tiene que hacer para cumplir
        # con cada paso de la interpolacion.
        for index, value in enumerate(self.values):
            pilas.mundo.tweener.addTweenNoArgs(target, function=function, 
                    initial_value=fist_value,
                    value=value, 
                    tweenDelay=self.delay * 1000.0 + (index * step),
                    tweenTime=step)
            # El siguiente valor inicial sera el que ha alcanzado.
            fist_value = value



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar
 
import math
import pilas

class Comportamiento:
    "Representa un comportamiento (estrategia) que se puede anexar a un actor."
    
    def iniciar(self, receptor):
        "Se invoca cuando se anexa el comportamiento a un actor."
        self.receptor = receptor

    def actualizar(self):
        """Actualiza el comportamiento en un instante dado.

        Si este metodo retorna True entonces el actor dejará
        de ejecutar este comportamiento."""
        pass

    def terminar(self):
        pass


class Girar(Comportamiento):
    "Hace girar constantemente al actor respecto de su eje de forma relativa."

    def __init__(self, delta, velocidad):
        self.delta = delta

        if delta > 0:
            self.velocidad = velocidad
        else:
            self.velocidad = -velocidad


    def iniciar(self, receptor):
        "Define el angulo inicial."
        self.receptor = receptor
        self.angulo_final = (receptor.rotacion + self.delta) % 360

    def actualizar(self):
        self.receptor.rotacion += self.velocidad

        delta = abs(self.receptor.rotacion - self.angulo_final)

        if delta <= abs(self.velocidad):
            self.receptor.rotacion = self.angulo_final
            return True


class Avanzar(Comportamiento):
    "Desplaza al actor en la dirección y sentido indicado por una rotación."

    def __init__(self, pasos, velocidad=5):
        self.pasos = abs(pasos)
        self.velocidad = velocidad

    def iniciar(self, receptor):
        self.receptor = receptor
        rotacion_en_radianes = math.radians(-receptor.rotacion)
        self.dx = math.cos(rotacion_en_radianes)
        self.dy = math.sin(rotacion_en_radianes)

    def actualizar(self):
        salir = False

        if self.pasos - self.velocidad < 0:
            avance = self.pasos
            salir = True
        else:
            avance = self.velocidad

        self.pasos -= avance
        self.receptor.x += self.dx * avance
        self.receptor.y += self.dy * avance

        if salir:
            return True

class CambiarColor(Comportamiento):
    "Llama a un metodo para cambiar el color de un actor."

    def __init__(self, nuevo_color):
        self.nuevo_color = nuevo_color

    def iniciar(self, receptor):
        self.receptor = receptor

    def actualizar(self):
        self.receptor.color = self.nuevo_color
        return True



class BajarLapiz(Comportamiento):

    def __init__(self):
        pass

    def iniciar(self, receptor):
        self.receptor = receptor

    def actualizar(self):
        self.receptor.pizarra.bajar_lapiz()
        return True

class SubirLapiz(Comportamiento):

    def __init__(self):
        pass

    def iniciar(self, receptor):
        self.receptor = receptor

    def actualizar(self):
        self.receptor.pizarra.levantar_lapiz()
        return True




# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

disponibles = []

try:
    from PySFML import sf
    disponibles.append('sfml')
except ImportError:
    pass

try:
    import pygame
    disponibles.append('pygame')
except ImportError:
    pass


# Incorpora todo el contenido de los motores, pero
# lo hace fuera del bloque try except para hacer
# fluir las excepciones.
if 'sfml' in disponibles:
    from motor_sfml import *

if 'pygame' in disponibles:
    from motor_pygame import *



import pilas

pilas.iniciar()

mono = pilas.actores.Mono()
mono.escala = 1
mono.rotacion = 0

def girar():
    mono.rotacion = mono.rotacion + 1
    return True

pilas.mundo.agregar_tarea(0, girar)

pilas.mundo.camara.x = pilas.interpolar([200, 0], duracion=10)
pilas.mundo.camara.y = pilas.interpolar([200, 0], duracion=10)

pilas.avisar("Moviendo la camara...")
pilas.ejecutar()



# pyTweener
#
# Tweening functions for python
#
# Heavily based on caurina Tweener: http://code.google.com/p/tweener/
#
# Released under M.I.T License - see above url
# Python version by Ben Harling 2009 
import math

class Tweener:
    def __init__(self, duration = 0.5, tween = None):
        """Tweener
        This class manages all active tweens, and provides a factory for
        creating and spawning tween motions."""
        self.currentTweens = []
        self.defaultTweenType = tween or Easing.Linear.easeNone
        self.defaultDuration = duration or 1.0
 
    def hasTweens(self):
        return len(self.currentTweens) > 0
 
    def addTweenNoArgs(self, obj, function, initial_value, value, **kwargs):
        "Similar a addTween, solo que se especifica la funcion y el valor de forma explicita."
        args = {function: value, 'initial_value': initial_value}

        if "tweenTime" in kwargs:
            t_time = kwargs.pop("tweenTime")
        else: t_time = self.defaultDuration
 
        if "tweenType" in kwargs:
            t_type = kwargs.pop("tweenType")
        else: t_type = self.defaultTweenType
 
        if "onCompleteFunction" in kwargs:
            t_completeFunc = kwargs.pop("onCompleteFunction")
        else: t_completeFunc = None
 
        if "onUpdateFunction" in kwargs:
            t_updateFunc = kwargs.pop("onUpdateFunction")
        else: t_updateFunc = None
 
        if "tweenDelay" in kwargs:
            t_delay = kwargs.pop("tweenDelay")
        else: t_delay = 0

        if kwargs:
            raise ValueError("No puede llamar a esta funcion con argumentos nombrados, use addTween en su lugar.")

        tw = Tween(obj, t_time, t_type, t_completeFunc, t_updateFunc, t_delay, **args)
        if tw:    
            self.currentTweens.append( tw )
        return tw

    def addTween(self, obj, **kwargs):
        """ addTween( object, **kwargs) -> tweenObject or False
 
            Example:
            tweener.addTween( myRocket, throttle=50, setThrust=400, tweenTime=5.0, tweenType=tweener.OUT_QUAD )
 
            You must first specify an object, and at least one property or function with a corresponding
            change value. The tween will throw an error if you specify an attribute the object does
            not possess. Also the data types of the change and the initial value of the tweened item
            must match. If you specify a 'set' -type function, the tweener will attempt to get the
            starting value by call the corresponding 'get' function on the object. If you specify a 
            property, the tweener will read the current state as the starting value. You add both 
            functions and property changes to the same tween.
 
            in addition to any properties you specify on the object, these keywords do additional
            setup of the tween.
 
            tweenTime = the duration of the motion
            tweenType = one of the predefined tweening equations or your own function
            onCompleteFunction = specify a function to call on completion of the tween
            onUpdateFunction = specify a function to call every time the tween updates
            tweenDelay = specify a delay before starting.
            """
        if "tweenTime" in kwargs:
            t_time = kwargs.pop("tweenTime")
        else: t_time = self.defaultDuration
 
        if "tweenType" in kwargs:
            t_type = kwargs.pop("tweenType")
        else: t_type = self.defaultTweenType
 
        if "onCompleteFunction" in kwargs:
            t_completeFunc = kwargs.pop("onCompleteFunction")
        else: t_completeFunc = None
 
        if "onUpdateFunction" in kwargs:
            t_updateFunc = kwargs.pop("onUpdateFunction")
        else: t_updateFunc = None
 
        if "tweenDelay" in kwargs:
            t_delay = kwargs.pop("tweenDelay")
        else: t_delay = 0
 
        tw = Tween( obj, t_time, t_type, t_completeFunc, t_updateFunc, t_delay, **kwargs )
        if tw:    
            self.currentTweens.append( tw )
        return tw
 
    def removeTween(self, tweenObj):
        if tweenObj in self.currentTweens:
            tweenObj.complete = True
            #self.currentTweens.remove( tweenObj )
 
    def getTweensAffectingObject(self, obj):
        """Get a list of all tweens acting on the specified object
        Useful for manipulating tweens on the fly"""
        tweens = []
        for t in self.currentTweens:
            if t.target is obj:
                tweens.append(t)
        return tweens
 
    def removeTweeningFrom(self, obj):
        """Stop tweening an object, without completing the motion
        or firing the completeFunction"""
        for t in self.currentTweens:
            if t.target is obj:
                t.complete = True
 
    def finish(self):
        #go to last frame for all tweens
        for t in self.currentTweens:
            t.update(t.duration)
        self.currentTweens = []
 
    def update(self, timeSinceLastFrame):
        removable = []
        for t in self.currentTweens:
            t.update(timeSinceLastFrame)

            if t.complete:
                removable.append(t)
                
        for t in removable:
            self.currentTweens.remove(t)
            
 
class Tween(object):
    def __init__(self, obj, tduration, tweenType, completeFunction, updateFunction, delay, **kwargs):
        """Tween object:
            Can be created directly, but much more easily using Tweener.addTween( ... )
            """
        #print obj, tduration, kwargs
        self.duration = tduration
        self.delay = delay
        self.target = obj
        self.tween = tweenType
        self.tweenables = kwargs
        self.delta = 0
        self.completeFunction = completeFunction
        self.updateFunction = updateFunction
        self.complete = False
        self.tProps = []
        self.tFuncs = []
        self.paused = self.delay > 0
        self.decodeArguments()
 
    def decodeArguments(self):
        """Internal setup procedure to create tweenables and work out
           how to deal with each"""
 
        if len(self.tweenables) == 0:
            # nothing to do 
            print "TWEEN ERROR: No Tweenable properties or functions defined"
            self.complete = True
            return

        assert(len(self.tweenables) == 2)

        initial_value = self.tweenables.pop('initial_value')


        for k, v in self.tweenables.items():
 
        # check that its compatible
            if not hasattr( self.target, k):
                print "TWEEN ERROR: " + str(self.target) + " has no function " + k
                self.complete = True
                break
 
            prop = func = False
            startVal = 0
            newVal = v
 
            try:
                startVal = self.target.__dict__[k]
                prop = k
                propName = k
 
            except:
                func = getattr( self.target, k)
                funcName = k
 
            if func:
                try:
                    getFunc = getattr(self.target, funcName.replace("set", "get") )
                    startVal = getFunc()
                    print getfunc
                except:
                    # no start value, assume its 0
                    # but make sure the start and change
                    # dataTypes match :)
                    startVal = newVal * 0

                startVal = initial_value
                tweenable = Tweenable( startVal, newVal - startVal)    
                newFunc = [ k, func, tweenable]
 
                #setattr(self, funcName, newFunc[2])
                self.tFuncs.append( newFunc )
 
 
            if prop:
                tweenable = Tweenable( startVal, newVal - startVal)    
                newProp = [ k, prop, tweenable]
                self.tProps.append( newProp )  
 
        """
        for k, v in self.tweenables.items():
 
        # check that its compatible
            if not hasattr( self.target, k):
                print "TWEEN ERROR: " + str(self.target) + " has no function " + k
                self.complete = True
                break
 
            prop = func = False
            startVal = 0
            newVal = v
 
            try:
                startVal = self.target.__dict__[k]
                prop = k
                propName = k
 
            except:
                func = getattr( self.target, k)
                funcName = k
 
            if func:
                try:
                    getFunc = getattr(self.target, funcName.replace("set", "get") )
                    startVal = getFunc()
                    print getfunc
                except:
                    # no start value, assume its 0
                    # but make sure the start and change
                    # dataTypes match :)
                    startVal = newVal * 0
                tweenable = Tweenable( startVal, newVal - startVal)    
                newFunc = [ k, func, tweenable]
 
                #setattr(self, funcName, newFunc[2])
                self.tFuncs.append( newFunc )
 
 
            if prop:
                tweenable = Tweenable( startVal, newVal - startVal)    
                newProp = [ k, prop, tweenable]
                self.tProps.append( newProp )  
        """ 

 
    def pause( self, numSeconds=-1 ):
        """Pause this tween
            do tween.pause( 2 ) to pause for a specific time
            or tween.pause() which pauses indefinitely."""
        self.paused = True
        self.delay = numSeconds
 
    def resume( self ):
        """Resume from pause"""
        if self.paused:
            self.paused=False
 
    def update(self, ptime):
        """Update this tween with the time since the last frame
            if there is an update function, it is always called
            whether the tween is running or paused"""
            
        if self.complete:
            return
        
        if self.paused:
            if self.delay > 0:
                self.delay = max( 0, self.delay - ptime )
                if self.delay == 0:
                    self.paused = False
                    self.delay = -1
                if self.updateFunction:
                    self.updateFunction()
            return
 
        self.delta = min(self.delta + ptime, self.duration)
 

        for propName, prop, tweenable in self.tProps:
            self.target.__dict__[prop] = self.tween( self.delta, tweenable.startValue, tweenable.change, self.duration )
        for funcName, func, tweenable in self.tFuncs:
            func( self.tween( self.delta, tweenable.startValue, tweenable.change, self.duration ) )
 
 
        if self.delta == self.duration:
            self.complete = True
            if self.completeFunction:
                self.completeFunction()
 
        if self.updateFunction:
            self.updateFunction()
 
 
 
    def getTweenable(self, name):
        """Return the tweenable values corresponding to the name of the original
        tweening function or property. 
 
        Allows the parameters of tweens to be changed at runtime. The parameters
        can even be tweened themselves!
 
        eg:
 
        # the rocket needs to escape!! - we're already moving, but must go faster!
        twn = tweener.getTweensAffectingObject( myRocket )[0]
        tweenable = twn.getTweenable( "thrusterPower" )
        tweener.addTween( tweenable, change=1000.0, tweenTime=0.4, tweenType=tweener.IN_QUAD )
 
        """
        ret = None
        for n, f, t in self.tFuncs:
            if n == name:
                ret = t
                return ret
        for n, p, t in self.tProps:
            if n == name:
                ret = t
                return ret
        return ret
 
    def Remove(self):
        """Disables and removes this tween
            without calling the complete function"""
        self.complete = True

 
class Tweenable:
    def __init__(self, start, change):
        """Tweenable:
            Holds values for anything that can be tweened
            these are normally only created by Tweens"""
        self.startValue = start
        self.change = change


"""Robert Penner's easing classes ported over from actionscript by Toms Baugis (at gmail com).
There certainly is room for improvement, but wanted to keep the readability to some extent.

================================================================================
 Easing Equations
 (c) 2003 Robert Penner, all rights reserved. 
 This work is subject to the terms in
 http://www.robertpenner.com/easing_terms_of_use.html.
================================================================================

TERMS OF USE - EASING EQUATIONS

Open source under the BSD License.

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of the author nor the names of contributors may be used
      to endorse or promote products derived from this software without specific
      prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
class Easing:
    class Back:
        @staticmethod
        def easeIn(t, b, c, d, s = 1.70158):
            t = t / d
            return c * t**2 * ((s+1) * t - s) + b

        @staticmethod
        def easeOut (t, b, c, d, s = 1.70158):
            t = t / d - 1
            return c * (t**2 * ((s + 1) * t + s) + 1) + b

        @staticmethod
        def easeInOut (t, b, c, d, s = 1.70158):
            t = t / (d * 0.5)
            s = s * 1.525
            
            if t < 1:
                return c * 0.5 * (t**2 * ((s + 1) * t - s)) + b

            t = t - 2
            return c / 2 * (t**2 * ((s + 1) * t + s) + 2) + b

    class Bounce:
        @staticmethod
        def easeOut (t, b, c, d):
            t = t / d
            if t < 1 / 2.75:
                return c * (7.5625 * t**2) + b
            elif t < 2 / 2.75:
                t = t - 1.5 / 2.75
                return c * (7.5625 * t**2 + 0.75) + b
            elif t < 2.5 / 2.75:
                t = t - 2.25 / 2.75
                return c * (7.5625 * t**2 + .9375) + b
            else:
                t = t - 2.625 / 2.75
                return c * (7.5625 * t**2 + 0.984375) + b

        @staticmethod
        def easeIn (t, b, c, d):
            return c - Easing.Bounce.easeOut(d-t, 0, c, d) + b

        @staticmethod
        def easeInOut (t, b, c, d):
            if t < d * 0.5:
                return Easing.Bounce.easeIn (t * 2, 0, c, d) * .5 + b

            return Easing.Bounce.easeOut (t * 2 -d, 0, c, d) * .5 + c*.5 + b


        
    class Circ:
        @staticmethod
        def easeIn (t, b, c, d):
            t = t / d
            return -c * (math.sqrt(1 - t**2) - 1) + b

        @staticmethod
        def easeOut (t, b, c, d):
            t = t / d - 1
            return c * math.sqrt(1 - t**2) + b

        @staticmethod
        def easeInOut (t, b, c, d):
            t = t / (d * 0.5)
            if t < 1:
                return -c * 0.5 * (math.sqrt(1 - t**2) - 1) + b
            
            t = t - 2
            return c*0.5 * (math.sqrt(1 - t**2) + 1) + b


    class Cubic:
        @staticmethod
        def easeIn (t, b, c, d):
            t = t / d
            return c * t**3 + b

        @staticmethod
        def easeOut (t, b, c, d):
            t = t / d - 1
            return c * (t**3 + 1) + b

        @staticmethod
        def easeInOut (t, b, c, d):
            t = t / (d * 0.5)
            if t < 1:
                return c * 0.5 * t**3 + b
            
            t = t - 2
            return c * 0.5 * (t**3 + 2) + b


    class Elastic:
        @staticmethod
        def easeIn (t, b, c, d, a = 0, p = 0):
            if t==0: return b

            t = t / d            
            if t == 1: return b+c
            
            if not p: p = d * .3;

            if not a or a < abs(c):
                a = c
                s = p / 4
            else:
                s = p / (2 * math.pi) * math.asin(c / a)
            
            t = t - 1            
            return - (a * math.pow(2, 10 * t) * math.sin((t*d-s) * (2 * math.pi) / p)) + b


        @staticmethod
        def easeOut (t, b, c, d, a = 0, p = 0):
            if t == 0: return b
            
            t = t / d
            if (t == 1): return b + c
            
            if not p: p = d * .3;

            if not a or a < abs(c):
                a = c
                s = p / 4
            else:
                s = p / (2 * math.pi) * math.asin(c / a)
                
            return a * math.pow(2,-10 * t) * math.sin((t * d - s) * (2 * math.pi) / p) + c + b


        @staticmethod
        def easeInOut (t, b, c, d, a = 0, p = 0):
            if t == 0: return b
            
            t = t / (d * 0.5)
            if t == 2: return b + c
            
            if not p: p = d * (.3 * 1.5)

            if not a or a < abs(c):
                a = c
                s = p / 4
            else:
                s = p / (2 * math.pi) * math.asin(c / a)
                
            if (t < 1):
                t = t - 1
                return -.5 * (a * math.pow(2, 10 * t) * math.sin((t * d - s) * (2 * math.pi) / p)) + b
                
            t = t - 1
            return a * math.pow(2, -10 * t) * math.sin((t * d - s) * (2 * math.pi) / p) * .5 + c + b


    class Expo:
        @staticmethod
        def easeIn(t, b, c, d):
            if t == 0:
                return b
            else:
                return c * math.pow(2, 10 * (t / d - 1)) + b - c * 0.001

        @staticmethod
        def easeOut(t, b, c, d):
            if t == d:
                return b + c
            else:
                return c * (-math.pow(2, -10 * t / d) + 1) + b

        @staticmethod
        def easeInOut(t, b, c, d):
            if t==0:
                return b
            elif t==d:
                return b+c

            t = t / (d * 0.5)
            
            if t < 1:
                return c * 0.5 * math.pow(2, 10 * (t - 1)) + b
            
            return c * 0.5 * (-math.pow(2, -10 * (t - 1)) + 2) + b


    class Linear:
        @staticmethod
        def easeNone(t, b, c, d):
            return c * t / d + b

        @staticmethod
        def easeIn(t, b, c, d):
            return c * t / d + b

        @staticmethod
        def easeOut(t, b, c, d):
            return c * t / d + b

        @staticmethod
        def easeInOut(t, b, c, d):
            return c * t / d + b


    class Quad:
        @staticmethod
        def easeIn (t, b, c, d):
            t = t / d
            return c * t**2 + b

        @staticmethod
        def easeOut (t, b, c, d):
            t = t / d
            return -c * t * (t-2) + b

        @staticmethod
        def easeInOut (t, b, c, d):
            t = t / (d * 0.5)
            if t < 1:
                return c * 0.5 * t**2 + b
            
            t = t - 1
            return -c * 0.5 * (t * (t - 2) - 1) + b


    class Quart:
        @staticmethod
        def easeIn (t, b, c, d):
            t = t / d
            return c * t**4 + b

        @staticmethod
        def easeOut (t, b, c, d):
            t = t / d - 1
            return -c * (t**4 - 1) + b

        @staticmethod
        def easeInOut (t, b, c, d):
            t = t / (d * 0.5)
            if t < 1:
                return c * 0.5 * t**4 + b
            
            t = t - 2
            return -c * 0.5 * (t**4 - 2) + b

    
    class Quint:
        @staticmethod
        def easeIn (t, b, c, d):
            t = t / d
            return c * t**5 + b

        @staticmethod
        def easeOut (t, b, c, d):
            t = t / d - 1
            return c * (t**5 + 1) + b

        @staticmethod
        def easeInOut (t, b, c, d):
            t = t / (d * 0.5)
            if t < 1:
                return c * 0.5 * t**5 + b
            
            t = t - 2
            return c * 0.5 * (t**5 + 2) + b

    class Sine:
        @staticmethod
        def easeIn (t, b, c, d):
            return -c * math.cos(t / d * (math.pi / 2)) + c + b

        @staticmethod
        def easeOut (t, b, c, d):
            return c * math.sin(t / d * (math.pi / 2)) + b

        @staticmethod
        def easeInOut (t, b, c, d):
            return -c * 0.5 * (math.cos(math.pi * t / d) - 1) + b


    class Strong:
        @staticmethod
        def easeIn(t, b, c, d):
            return c * (t/d)**5 + b

        @staticmethod
        def easeOut(t, b, c, d):
            return c * ((t / d - 1)**5 + 1) + b

        @staticmethod
        def easeInOut(t, b, c, d):
            t = t / (d * 0.5)
            
            if t < 1:
                return c * 0.5 * t**5 + b
            
            t = t - 2
            return c * 0.5 * (t**5 + 2) + b



class TweenTestObject:
    def __init__(self):
        self.pos = 20
        self.rot = 50
 
    def update(self):
        print self.pos, self.rot
 
    def setRotation(self, rot):
        self.rot = rot
 
    def getRotation(self):
        return self.rot
 
    def complete(self):
        print "I'm done tweening now mommy!"
 
 
if __name__=="__main__":
    import time
    T = Tweener()
    tst = TweenTestObject()
    mt = T.addTween( tst, setRotation=500.0, tweenTime=2.5, tweenType=T.OUT_QUAD, 
                      pos=-200, tweenDelay=0.4, onCompleteFunction=tst.complete, 
                      onUpdateFunction=tst.update )
    s = time.clock()
    changed = False
    while T.hasTweens():
        tm = time.clock()
        d = tm - s
        s = tm
        T.update( d )
        if mt.delta > 1.0 and not changed:
 
            tweenable = mt.getTweenable( "setRotation" )
 
            T.addTween( tweenable, change=-1000, tweenTime=0.7 )
            T.addTween( mt, duration=-0.2, tweenTime=0.2 )
            changed = True
        #print mt.duration,
        print tst.getRotation(), tst.pos
        time.sleep(0.06)
    print tst.getRotation(), tst.pos



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar


import pymunk


class Fisica:
    "Representa el motor de simulacion fisica."

    def __init__(self):
        pymunk.init_pymunk()

        self.espacio = pymunk.Space()
        self.definir_gravedad(0, -900)
        self._crear_suelos()

    def _crear_suelos(self):
        static_body = pymunk.Body(pymunk.inf, pymunk.inf)
        static_lines = [
                        # Borde inferior
                        pymunk.Segment(static_body, (-320.0, -240.0), (320.0, -240.0), 0.0),
                        # Borde izquierdo
                        pymunk.Segment(static_body, (-320.0, 240.0), (-320.0, -240.0), 0.0),
                        # Borde izquierdo
                        pymunk.Segment(static_body, (320.0, 240.0), (320.0, -240.0), 0.0),
                        # Borde superior
                        pymunk.Segment(static_body, (-320.0, 240.0), (320.0, 240.0), 0.0),
                        ]

        for line in static_lines:
            line.body.mass = 0
            line.body.moment = 0

        self.espacio.add_static(static_lines)


    def actualizar(self):
        self.espacio.step(1/50.0)

    def agregar(self, figura):
        self.espacio.add(figura)

    def eliminar(self, figura):
        try:
            self.espacio.remove(figura)
        except:
            try:
                # Intenta eliminar la figura como si fuera un objeto estatico.
                self.espacio.remove_static(figura)
            except:
                #TODO: quitar este silenciador, porque esta para cuando
                #      falla el manejo de grupos.
                pass


    def crear_figura_circulo(self, x, y, radio, masa=1, elasticidad=0.05, friccion=0.05):
        inertia = pymunk.moment_for_circle(masa, 0, radio, (0,0))
        body = pymunk.Body(masa, inertia)
        body.position = x, y
        shape = pymunk.Circle(body, radio)
        shape.elasticity = elasticidad
        shape.friction = friccion
        self.espacio.add(body, shape)
        return shape

    def crear_figura_cuadrado(self, x, y, ancho, masa=1, elasticidad=0.05, friccion=0.0):
        inertia = pymunk.moment_for_box(masa, ancho, ancho)
        body = pymunk.Body(masa, inertia)
        body.position = x, y
        r = ancho 
        shape = pymunk.Poly(body,  [(-r, -r), (-r, r), (r, r), (r, -r)], (0, 0))
        shape.elasticity = elasticidad
        shape.friction = friccion
        self.espacio.add(body, shape)
        return shape

    def definir_gravedad(self, x=0, y=-900):
        self.espacio.gravity = (x, y)

    def crear_cuadrado_estatico(self, x, y, ancho):
        static_body = pymunk.Body(pymunk.inf, pymunk.inf)
        static_lines = [
                        # Borde inferior
                        pymunk.Segment(static_body, (x, y-ancho), (x+ancho, y-ancho), 0.0),
                        # Borde derecho
                        pymunk.Segment(static_body, (x+ancho, y), (x+ancho, y-ancho), 0.0),
                        # Borde izquierdo
                        pymunk.Segment(static_body, (x, y), (x, y-ancho), 0.0),
                        # Borde superior
                        pymunk.Segment(static_body, (x, y), (x + ancho, y), 0.0),
                        ]

        for line in static_lines:
            line.elasticity = 0.0

        self.espacio.add_static(static_lines)
        return static_lines

    def empujar_figura(self, figura, x, y):
        "Empuja una figura fisica en la direccion que indica x e y."
        figura.body.apply_impulse((x, y))

fisica = Fisica()



import pilas

pilas.iniciar()
mono = pilas.actores.Mono()
mono.x = 0
mono.y = 0
mono.rotacion = pilas.interpolar(360, duracion=3)
mono.escala = pilas.interpolar(2, duracion=3)
mono.x = pilas.interpolar(320, duracion=3)
mono.y = pilas.interpolar(240, duracion=3)

pilas.avisar("Un ejemplo de interpolacion en dos dimensiones.")
pilas.ejecutar()




# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas.utils

class Estudiante:
    """Representa la habilidad de poder realizar habiliadades y comportamientos."""

    def __init__(self):
        self.habilidades = []
        self.comportamiento_actual = None
        self.comportamientos = []
        self.repetir_comportamientos_por_siempre = False

    def aprender(self, classname, *k, **w):
        "Comienza a realizar una habilidad indicada por parametros."
        objeto_habilidad = classname(self, *k, **w)
        self.habilidades.append(objeto_habilidad)

    def hacer_luego(self, comportamiento, repetir_por_siempre=False):
        """Define un nuevo comportamiento para realizar al final.
        
        Los actores pueden tener una cadena de comportamientos, este
        metodo agrega el comportamiento al final de la cadena.
        """
        
        self.comportamientos.append(comportamiento)
        self.repetir_comportamientos_por_siempre = repetir_por_siempre

    def hacer(self, comportamiento):
        "Define el comportamiento para el actor de manera inmediata."
        self.comportamientos.append(comportamiento)
        self._adoptar_el_siguiente_comportamiento()

    def eliminar_habilidades(self):
        "Elimina todas las habilidades asociadas al actor."
        for h in self.habilidades:
            h.eliminar()

    def eliminar_comportamientos(self):
        "Elimina todos los comportamientos que tiene que hacer el actor."
        for c in self.comportamientos:
            c.eliminar()

    def actualizar_habilidades(self):
        for h in self.habilidades:
            h.actualizar()

    def actualizar_comportamientos(self):
        termina = None

        if self.comportamiento_actual:
            termina = self.comportamiento_actual.actualizar()

            if termina:
                if self.repetir_comportamientos_por_siempre:
                    self.comportamientos.append(self.comportamiento_actual)
                self._adoptar_el_siguiente_comportamiento()
        else:
            self._adoptar_el_siguiente_comportamiento()

    def _adoptar_el_siguiente_comportamiento(self):
        if self.comportamientos:
            self.comportamiento_actual = self.comportamientos.pop(0)
            self.comportamiento_actual.iniciar(self)
        else:
            self.comportamiento_actual = None

class BaseActor(object, Estudiante):
    """Define la funciondad abstracta de un actor.

    La responsabilidad de esta clase es:

        - Controlar la posicion y el area que ocupa en pantalla.

    La clase que extienda a este debe tener
    métodos para dar el tamano del sprite, informar y alterar
    la posicion en pantalla.
    """

    def __init__(self, x=0, y=0):
        self._definir_centro_del_actor()
        Estudiante.__init__(self)

        self.x = x
        self.y = y

        # Define el nivel de lejania respecto del observador.
        self.z = 0
        self._espejado = False
        self.radio_de_colision = 10
        pilas.actores.utils.insertar_como_nuevo_actor(self)
        self._transparencia = 0

    def _definir_centro_del_actor(self):
        "Hace que el eje de posición del actor sea el centro de la imagen."
        size = self.obtener_area()
        self.definir_centro(size[0]/2, size[1]/2)

    def get_x(self):
        x, y = self.obtener_posicion()
        return x

    @pilas.utils.interpolable
    def set_x(self, x):
        self.definir_posicion(x, self.y)

    def get_z(self):
        return self._z

    @pilas.utils.interpolable
    def set_z(self, z):
        self._z = z
        pilas.actores.utils.ordenar_actores_por_valor_z()

    @pilas.utils.interpolable
    def set_y(self, y):
        self.definir_posicion(self.x, y)

    def get_y(self):
        x, y = self.obtener_posicion()
        return y

    @pilas.utils.interpolable
    def set_scale(self, s):
        if s <= 0:
            return

        ultima_escala = self.obtener_escala()

        # Se hace la siguiente regla de 3 simple:
        #
        #  ultima_escala          self.radio_de_colision
        #  s                      ?

        self.definir_escala(s)
        self.radio_de_colision = (s * self.radio_de_colision) / ultima_escala

    def get_scale(self):
        return self.obtener_escala()

    def get_rotation(self):
        return self.obtener_rotacion()

    @pilas.utils.interpolable
    def set_rotation(self, x):
        self.definir_rotacion(x)

    def get_espejado(self):
        return self._espejado

    def set_espejado(self, nuevo_valor):
        if self._espejado != nuevo_valor:
            self._espejado = nuevo_valor
            self.FlipX(self._espejado)

    @pilas.utils.interpolable
    def set_transparencia(self, nuevo_valor):
        self._transparencia = nuevo_valor
        self.definir_transparencia(nuevo_valor)

    def get_transparencia(self):
        return self._transparencia

    def get_imagen(self):
        return self.obtener_imagen()

    def set_imagen(self, imagen):
        if isinstance(imagen, str):
            imagen = pilas.imagenes.cargar(imagen)

        self.definir_imagen(imagen)
        

    espejado = property(get_espejado, set_espejado, doc="Indica si se tiene que invertir horizonaltamente la imagen del actor.")
    z = property(get_z, set_z, doc="Define lejania respecto del observador.")
    x = property(get_x, set_x, doc="Define la posición horizontal.")
    y = property(get_y, set_y, doc="Define la posición vertical.")
    rotacion = property(get_rotation, set_rotation, doc="Angulo de rotación (en grados, de 0 a 360)")
    escala = property(get_scale, set_scale, doc="Escala de tamaño, 1 es normal, 2 al doble de tamaño etc...)")
    transparencia = property(get_transparencia, set_transparencia, doc="Define el nivel de transparencia, 0 indica opaco y 100 la maxima transparencia.")
    imagen = property(get_imagen, set_imagen, doc="Define la imagen a mostrar.")

    def eliminar(self):
        "Elimina el actor de la lista de actores que se imprimen en pantalla."
        pilas.actores.utils.eliminar_un_actor(self)
        self.eliminar_habilidades()
        self.eliminar_comportamientos()

    def actualizar(self):
        """Actualiza el estado del actor. 
        
        Este metodo se llama una vez por frame, y generalmente se redefine
        en alguna subclase."""
        pass

    def __cmp__(self, otro_actor):
        """Compara dos actores para determinar cual esta mas cerca de la camara.

        Este metodo se utiliza para ordenar los actores antes de imprimirlos
        en pantalla. De modo tal que un usuario pueda seleccionar que
        actores se ven mas arriba de otros cambiando los valores de
        los atributos `z`."""

        if otro_actor.z >= self.z:
            return 1
        else:
            return -1


    def get_izquierda(self):
        return self.x - self.obtener_ancho()/2

    @pilas.utils.interpolable
    def set_izquierda(self, x):
        self.x = x + self.obtener_ancho()/2

    izquierda = property(get_izquierda, set_izquierda)

    def get_abajo(self):
        return self.y - self.obtener_alto()/2

    @pilas.utils.interpolable
    def set_abajo(self, y):
        self.y = y + self.obtener_alto()/2

    abajo = property(get_abajo, set_abajo)

    def get_derecha(self):
        return self.x + self.obtener_ancho()/2

    @pilas.utils.interpolable
    def set_derecha(self, x):
        self.x = x - self.obtener_ancho()/2

    derecha = property(get_derecha, set_derecha)

    def get_arriba(self):
        return self.y + self.obtener_alto()/2

    @pilas.utils.interpolable
    def set_arriba(self, y):
        self.y = y - self.obtener_alto()/2

    arriba = property(get_arriba, set_arriba)

    def colisiona_con_un_punto(self, x, y):
        "Determina si un punto colisiona con el area del actor."

        w, h = self.obtener_ancho(), self.obtener_alto()
        left, right = self.x - w/2 , self.x + w/2
        top, bottom = self.y - h/2,  self.y + h/2
        return left < x < right and top < y < bottom



import pilas

pilas.iniciar()

pizarra = pilas.actores.Pizarra()


# Selecciona solamente a los atributos que parecen colores.
colores = [color for color in dir(pilas.colores) if not '_' in color and not color == 'pilas']

# Por cada color imprime su nombre y un rectangulo de su color.
for indice, color in enumerate(colores):
    dy = 200 - indice * 23
    pizarra.definir_color(getattr(pilas.colores, color))
    pizarra.dibujar_rectangulo(80, dy, 20, 20)

    pizarra.definir_color(pilas.colores.negro)
    pizarra.escribir(color, -100, dy - 15, tamano=10)

pilas.ejecutar()



"""
This file is part of the 'Elements' Project
Elements is a 2D Physics API for Python (supporting Box2D2)

Copyright (C) 2008, The Elements Team, <elements@linuxuser.at>

Home:  http://elements.linuxuser.at
IRC:   #elements on irc.freenode.org

Code:  http://www.assembla.com/wiki/show/elements
       svn co http://svn2.assembla.com/svn/elements                     

License:  GPLv3 | See LICENSE for the full text
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.              
"""
INPUT_METERS = 0
INPUT_PIXELS = 1

CALLBACK_CONTACT_ADD = 0
CALLBACK_CONTACT_PERSIST = 1
CALLBACK_CONTACT_REMOVE  = 2

CALLBACK_DRAWING_START  = 3
CALLBACK_DRAWING_END    = 4

FLT_EPSILON = 1.192092896e-07



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pilas

class Escena:
    "Escena abstracta."

    def __init__(self):
        pilas.mundo.definir_escena(self)

    def iniciar(self):
        pass

    def actualizar(self):
        pass

    def dibujar(self, aplicacion):
        pass


class Normal(Escena):
    "Representa la escena inicial mas simple."

    def __init__(self, color_de_fondo=None):
        Escena.__init__(self)
        self.fondo = color_de_fondo or pilas.colores.grisclaro

    def dibujar(self, aplicacion):
        pilas.motor.pintar(self.fondo)



"""
This file is part of the 'Elements' Project
Elements is a 2D Physics API for Python (supporting Box2D2)

Copyright (C) 2008, The Elements Team, <elements@linuxuser.at>

Home:  http://elements.linuxuser.at
IRC:   #elements on irc.freenode.org

Code:  http://www.assembla.com/wiki/show/elements
       svn co http://svn2.assembla.com/svn/elements                     

License:  GPLv3 | See LICENSE for the full text
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.              
"""
from locals import *

class Camera:
    """ The Camera class. We will see :)
        Please also see: http://www.assembla.com/spaces/elements/tickets/31
        
        This class currently handles:
        - Scaling factor
        - Screen Offset from the World Coordinate System
        
        Inputs from the user have to be checked for them.
        - Places to check for it: elements.py, drawing.py, add_objects.py
        
    """
    scale_factor = 1.0          # All coords to the renderer are multiplied with the scale factor in elements.draw()
    track_body = None           # Body which means to be tracked. Offset is set at each elements.draw()
    
    def __init__(self, parent):
        self.parent = parent
            
    def track(self, body):
        """ Start tracking a specific body
        """
        self.track_body = body

    def track_stop(self):
        """ Stop tracking a body
        """
        self.track_body = None
            
    def center(self, pos, screenCoord=True, stopTrack=True):
        """ Centers the camera at given screen coordinates -- in pixel
            Typical call: world.camera.center(event.pos)
            
            Problem: Works ONLY WITH screenCoord now!
        """        
        x, y = pos

        x -= self.parent.display_width / 2
        y -= self.parent.display_height  / 2

        if screenCoord:
            x /= self.scale_factor
            y /= self.scale_factor

        # Set the offset
        self.inc_offset((x, y), screenCoord, stopTrack)
        
    def set_offset(self, offset, screenCoord=True, stopTrack=True):
        """ Set an offset from the screen to the world cs 
            -- in screen (or world) coordinates and in pixel
        """
        # Stop tracking of an object
        if stopTrack: self.track_stop()

        # If screenCoords, we have to bring them to the world cs
        if screenCoord: x, y = self.parent.to_world(offset)
        else: x, y = offset

        self._set_offset((x/self.parent.ppm, y/self.parent.ppm))

    def inc_offset(self, offset, screenCoord=True, stopTrack=True):
        """ Increment an offset from the screen to the world cs -- in world coordinates and in pixel
        """
        # Stop tracking of an object
        if stopTrack: self.track_stop()

        # Get Current Offset
        x, y = self.parent.screen_offset_pixel
        dx, dy = offset

        # Bring the directions into the world coordinate system
        if screenCoord:
            if self.parent.inputAxis_x_left: dx *= -1
            if self.parent.inputAxis_y_down: dy *= -1
                
        # Set New Offset
        self._set_offset(((x+dx)/self.parent.ppm, (y+dy)/self.parent.ppm))
                
    def _set_offset(self, offset):
        """ Set the screen offset to the world coordinate system
            (using meters and the world coordinate system's orientation)
        """
        x, y = offset            
        self.parent.screen_offset = (x, y)
        self.parent.screen_offset_pixel = (x*self.parent.ppm, y*self.parent.ppm)
        
    def set_scale_factor(self, factor=1.0):
        """ Zoom factor for the renderer 1.0 = 1:1 (original)
        """
        self.scale_factor = factor
        
    def inc_scale_factor(self, factor=0.0):
        """ Increases the zooms for the renderer a given factor
        """
        self.scale_factor += factor
        
        


# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

from PySFML import sf
from pilas.simbolos import *
import cairo
import pilas
from pilas import eventos

ANCHO = 640
ALTO = 480

class pySFML:
    import pilas.baseactor

    class Color(sf.Color):

        def obtener_componentes(self):
            return self.b, self.g, self.r

    class Canvas:
        "Representa una superficie sobre la que se puede dibujar usando cairo."

        def __init__(self):
            self.surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, ANCHO, ALTO)
            self.image = sf.Image()
            self.context = cairo.Context(self.surface)
            self.actualizar()

        def actualizar(self):
            self.image.LoadFromPixels(ANCHO, ALTO, self.surface.get_data())


    class Texto(sf.String, pilas.baseactor.BaseActor):
        """Representa un texto en pantalla.

        El texto tiene atributos como ``texto``, ``magnitud`` y ``color``.
        """

        def __init__(self, texto="None", x=0, y=0):
            sf.String.__init__(self, texto)
            self.color = pilas.colores.negro
            pilas.baseactor.BaseActor.__init__(self, x=x, y=y)

        def get_text(self):
            return self.GetText()

        def set_text(self, text):
            self.SetText(text)

        def get_size(self):
            return self.GetSize()

        def set_size(self, size):
            self.SetSize(size)
            self._set_central_axis()

        def _set_central_axis(self):
            rect = self.GetRect()
            size = (rect.GetWidth(), rect.GetHeight())
            self.SetCenter(size[0]/2, size[1]/2)

        def obtener_posicion(self):
            x, y = self.GetPosition()
            return x, -y

        def definir_posicion(self, x, y):
            self.SetPosition(x, -y)

        def get_color(self):
            return self.GetColor()

        def set_color(self, k):
            self.SetColor(k)

        texto = property(get_text, set_text, doc="El texto que se tiene que mostrar.")
        magnitud = property(get_size, set_size, doc="El tamaño del texto.")
        color = property(get_color, set_color, doc="Color del texto.")

        def dibujar(self, aplicacion):
            aplicacion.Draw(self)

        def colisiona_con_un_punto(self, x, y):
            return False

        def obtener_ancho(self):
            rect = self.GetRect()
            return rect.GetWidth()

        def obtener_alto(self):
            rect = self.GetRect()
            return rect.GetHeight()

        def obtener_area(self):
            return self.obtener_ancho(), self.obtener_alto()

        def definir_centro(self, x, y):
            self.SetCenter(x, y)

    class Grilla:
        """Representa una grilla de imagenes con varios cuadros de animación.

        Una grilla es un objeto que se tiene que inicializar con la ruta
        a una imagen, la cantidad de columnas y filas.

        Por ejemplo, si tenemos una grilla con 2 columnas y 3 filas
        podemos asociarla a un actor de la siguiente manera::

            grilla = pilas.imagenes.Grilla("animacion.png", 2, 3)
            grilla.asignar(actor)

        Entonces, a partir de ahora nuestro actor muestra solamente un
        cuadro de toda la grilla.

        Si quieres avanzar la animacion tienes que modificar el objeto
        grilla y asignarlo nuevamente al actor::

            grilla.avanzar()
            grilla.asignar(actor)
        """

        def __init__(self, ruta, columnas=1, filas=1):
            self.image = pilas.imagenes.cargar(ruta)
            self.cantidad_de_cuadros = columnas * filas
            self.columnas = columnas
            self.filas = filas
            self.cuadro_ancho = self.image.GetWidth() / columnas
            self.cuadro_alto = self.image.GetHeight() / filas
            self.sub_rect = sf.IntRect(0, 0, self.cuadro_ancho, self.cuadro_alto)
            self.definir_cuadro(0)

        def definir_cuadro(self, cuadro):
            self.cuadro = cuadro

            frame_col = cuadro % self.columnas
            frame_row = cuadro / self.columnas

            dx = frame_col * self.cuadro_ancho - self.sub_rect.Left
            dy = frame_row * self.cuadro_alto - self.sub_rect.Top

            self.sub_rect.Offset(dx, dy)

        def asignar(self, sprite):
            "Sets the sprite's image with animation state."

            sprite.SetImage(self.image)
            sprite.SetSubRect(self.sub_rect)
            sprite.SetCenter(self.cuadro_ancho / 2, self.cuadro_alto / 2)

        def avanzar(self):
            ha_reiniciado = False
            cuadro_actual = self.cuadro + 1

            if cuadro_actual >= self.cantidad_de_cuadros:
                cuadro_actual = 0
                ha_reiniciado = True

            self.definir_cuadro(cuadro_actual)
            return ha_reiniciado

        def obtener_cuadro(self):
            return self.cuadro

    class Sonido:

        def __init__(self, ruta):
            buff = sf.SoundBuffer()
            buff.LoadFromFile(ruta)
            self.buffer = buff
            self.sonido = sf.Sound(self.buffer)

        def reproducir(self):
            self.sonido.Play()
        
        def Play(self):
            self.reproducir()

    class Actor(pilas.baseactor.BaseActor, sf.Sprite):
        """Representa un objeto visible en pantalla, algo que se ve y tiene posicion.

        Un objeto Actor se tiene que crear siempre indicando la imagen, ya
        sea como una ruta a un archivo como con un objeto Image. Por ejemplo::

            protagonista = Actor("protagonista_de_frente.png")

        es equivalente a:

            imagen = pilas.imagenes.cargar("protagonista_de_frente.png")
            protagonista = Actor(imagen)

        Luego, na vez que ha sido ejecutada la sentencia aparecerá en el centro de
        la pantalla el nuevo actor para que pueda manipularlo. Por ejemplo
        alterando sus propiedades::

            protagonista.x = 100
            protagonista.scale = 2
            protagonista.rotation = 30


        Estas propiedades tambien se pueden manipular mediante
        interpolaciones. Por ejemplo, para aumentar el tamaño del
        personaje de 1 a 5 en 7 segundos::

            protagonista.scale = pilas.interpolar(1, 5, 7)

        Si creas un sprite sin indicar la imagen se cargará
        una por defecto.
        """

        def __init__(self, imagen="sin_imagen.png", x=0, y=0):

            if isinstance(imagen, str):
                imagen = pilas.imagenes.cargar(imagen)

            sf.Sprite.__init__(self, imagen)
            pilas.baseactor.BaseActor.__init__(self, x=x, y=y)
            

        def definir_imagen(self, imagen):
            self.SetImage(imagen)

        def obtener_imagen(self):
            return self.GetImage()

        def dibujar(self, aplicacion):
            aplicacion.Draw(self)

        
        def duplicar(self, **kv):
            duplicado = self.__class__()

            for clave in kv:
                setattr(duplicado, clave, kv[clave])

            return duplicado

        def obtener_ancho(self):
            return self.GetSize()[0]

        def obtener_alto(self):
            return self.GetSize()[1]

        def __str__(self):
            return "<%s en (%d, %d)>" %(self.__class__.__name__, self.x, self.y)

        def obtener_area(self):
            return self.obtener_ancho(), self.obtener_alto()

        def definir_centro(self, x, y):
            self.SetCenter(x, y)

        def obtener_posicion(self):
            x, y = self.GetPosition()
            return x, -y

        def definir_posicion(self, x, y):
            self.SetPosition(x, -y)

        def obtener_escala(self):
            return self.GetScale()[0]

        def definir_escala(self, s):
            self.SetScale(s, s)

        def definir_transparencia(self, nuevo_valor):
            nivel = min(255, 255 - (nuevo_valor*128) / 50)
            nivel = max(0, nivel)
            self.SetColor(sf.Color(255, 255, 255, int(nivel)))

        def obtener_rotacion(self):
            return (- self.GetRotation()) % 360

        def definir_rotacion(self, r):
            self.SetRotation(-r % 360)

    def __init__(self):
        # Se usan para calcular el dx y dy del movimiento
        # del mouse porque pySFML no lo reporta de forma relativa.
        self.mouse_x = 0
        self.mouse_y = 0

    def crear_ventana(self, ancho, alto, titulo):
        ventana = sf.RenderWindow(sf.VideoMode(ancho, alto), titulo)
        # Define que la coordenada (0, 0) sea el centro de la ventana.
        view = ventana.GetDefaultView()
        view.SetCenter(0, 0)
        self.input = ventana.GetInput()
        self.event = sf.Event()
        self.vista_de_camara = ventana.GetDefaultView()
        self.ventana = ventana
        return ventana

    def ocultar_puntero_del_mouse(self):
        self.ventana.ShowMouseCursor(False)

    def mostrar_puntero_del_mouse(self):
        self.ventana.ShowMouseCursor(True)

    def cerrar_ventana(self):
        self.ventana.Close()

    def dibujar_circulo(self, x, y, radio, color, color_borde):
        delta = radio / 2
        circulo = sf.Shape.Circle(0, 0, delta, color, 2, color_borde)
        circulo.SetCenter(0, 0)
        circulo.SetPosition(x, -y)
        self.ventana.Draw(circulo)

    def pulsa_tecla(self, tecla):
        "Indica si una tecla esta siendo pulsada en este instante."

        mapa = {
                IZQUIERDA: sf.Key.Left,
                DERECHA: sf.Key.Right,
                ARRIBA: sf.Key.Up,
                ABAJO: sf.Key.Down,
                BOTON: sf.Key.Space,
                SELECCION: sf.Key.Return,
                }

        return self.input.IsKeyDown(mapa[tecla])


    def centrar_ventana(self):
        "Coloca la ventana principal en el centro del escritorio."

        vm = sf.VideoMode(100, 100)

        # Obtiene la resolución del escritorio y la ventana.
        desktop_mode = vm.GetDesktopMode()
        w, h = self.ventana.GetWidth(), self.ventana.GetHeight()

        # Calcula cual debería la coordenada para centrar la ventana.
        to_x = desktop_mode.Width/2 - w/2
        to_y = desktop_mode.Height/2 - h/2

        self.ventana.SetPosition(to_x, to_y)

    def procesar_y_emitir_eventos(self):
        "Procesa todos los eventos que la biblioteca SFML pone en una cola."
        event = self.event

        while self.ventana.GetEvent(self.event):
            if event.Type == sf.Event.Closed:
                pilas.mundo.terminar()
            if event.Type == sf.Event.KeyPressed:
                self.procesar_evento_teclado(event)

                if event.Key.Code == sf.Key.Q:
                    pilas.mundo.terminar()

            elif event.Type == sf.Event.MouseMoved:
                # Notifica el movimiento del mouse con una señal

                x, y = event.MouseMove.X, event.MouseMove.Y

                if x > 0 and y > 0:
                    x, y = self.ventana.ConvertCoords(x, y)
                    y = -y

                    # Se asegura de los eventos de mouse esten siempre
                    # dentro de la ventana.
                    x = min(320, x)
                    y = max(y, -240)

                    dx = x - self.mouse_x
                    dy = y - self.mouse_y

                    self.mouse_x = x
                    self.mouse_y = y

                    eventos.mueve_mouse.send("ejecutar", x=x, y=y, dx=dx, dy=dy)

            elif event.Type == sf.Event.MouseButtonPressed:
                x, y = self.ventana.ConvertCoords(event.MouseButton.X, event.MouseButton.Y)
                eventos.click_de_mouse.send("ejecutar", button=event.MouseButton.Button, x=x, y=-y)
            elif event.Type == sf.Event.MouseButtonReleased:
                x, y = self.ventana.ConvertCoords(event.MouseButton.X, event.MouseMove.Y)
                eventos.termina_click.send("ejecutar", button=event.MouseButton.Button, x=x, y=-y)
            elif event.Type == sf.Event.MouseWheelMoved:
                eventos.mueve_rueda.send("ejecutar", delta=event.MouseWheel.Delta)

    def procesar_evento_teclado(self, event):
        eventos.pulsa_tecla.send("ejecutar", code=event.Key)

        if event.Key.Code == sf.Key.P:
            pilas.mundo.alternar_pausa()
        elif event.Key.Code == sf.Key.F12:
            pilas.mundo.alternar_modo_depuracion()
        elif event.Key.Code == sf.Key.Escape:
            eventos.pulsa_tecla_escape.send("ejecutar")

    def actualizar_pantalla(self):
        self.ventana.Display()

    def definir_centro_de_la_camara(self, x, y):
        view = self.ventana.GetDefaultView()
        view.SetCenter(x, y)

    def obtener_centro_de_la_camara(self):
        view = self.ventana.GetDefaultView()
        return view.GetCenter()

    def pintar(self, color):
        self.ventana.Clear(color)
            
    def cargar_sonido(self, ruta):
        return pySFML.Sonido(ruta)

    def cargar_imagen(self, ruta):
        image = sf.Image()
        image.LoadFromFile(ruta)
        return image

    def generar_imagen_cairo(self, imagen):
        """Retorna una superficie de cairo representando a la imagen.

        Esta funcion es util para pintar imagenes sobre una pizarra
        o el escenario de un videojuego.
        """
        import array

        pixels = array.array("B", imagen.GetPixels())

        w = imagen.GetWidth()
        h = imagen.GetHeight()

        return cairo.ImageSurface.create_for_data(pixels, cairo.FORMAT_RGB24, w, h)



"""
This file is part of the 'Elements' Project
Elements is a 2D Physics API for Python (supporting Box2D2)

Copyright (C) 2008, The Elements Team, <elements@linuxuser.at>

Home:  http://elements.linuxuser.at
IRC:   #elements on irc.freenode.org

Code:  http://www.assembla.com/wiki/show/elements
       svn co http://svn2.assembla.com/svn/elements                     

License:  GPLv3 | See LICENSE for the full text
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.              
"""
# Some Hex Tools
def hex2dec(hex):
    """ Convert and hex value in a decimal number
    """ 
    return int(hex, 16)

def hex2rgb(hex): 
    """ Convert a hex color (#123abc) in RGB ((r), (g), (b))
    """
    if hex[0:1] == '#': hex = hex[1:]; 
    return (hex2dec(hex[:2]), hex2dec(hex[2:4]), hex2dec(hex[4:6]))

def rgb2floats(rgb):
    """Convert a color in the RGB (0..255,0..255,0..255) format to the
       (0..1, 0..1, 0..1) float format
    """
    ret = []
    for c in rgb:
        ret.append(float(c) / 255)
    return ret

def point_in_poly(point, poly):
    #print ">", point, poly
    x, y = point
    n = len(poly)
    inside = False
    p1x,p1y = poly[0]
    for i in range(n+1):
        p2x,p2y = poly[i % n]
        if y > min(p1y,p2y):
            if y <= max(p1y,p2y):
                if x <= max(p1x,p2x):
                    if p1y != p2y:
                        xinters = (y-p1y)*(p2x-p1x)/(p2y-p1y)+p1x
                    if p1x == p2x or x <= xinters:
                        inside = not inside
        p1x,p1y = p2x,p2y
    return inside
    


# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

class Tareas:
    """Contenedor de tareas a ejecutar por tiempo.

    El Tareas es un planificador de tareas, permite que
    podamos ejecutar funciones y métodos luego de transcurrido
    el tiempo que queramos.

    Por ejemplo, si se quiere que el planificardor ejecute
    una función dentro de dos segundos podemos escribir:

        pilas.add_task(2, hola)

    o bien, especificando argumentos para esa función:

        pilas.add_task(4, hola, 'persona')

    La función que se especifique como segundo argumento
    tiene que retornar True o False. Si retorna True será
    colocada nuevamente en la cola de tareas una vez que se
    ejecute (esto es útil para crear bucles).
    """

    def __init__(self):
        self.tasks = []
        self.time_counter = 0

    def update(self, dt):
        "Actualiza los contadores de tiempo y ejecuta las tareas pendientes."
        self.time_counter += dt
        to_remove = []

        for t, dt, function, params in self.tasks:
            if self.time_counter > t:
                must_continue = function(*params)
                to_remove.append((t, dt, function, params))

                if must_continue:
                    self.agregar(dt, function, params)

        # Elimina de la lista de trabajos los que ya han sido ejecutados.
        if to_remove:
            for x in to_remove:
                self.tasks.remove(x)

    def agregar(self, time_out, function, params):
        "Agrega una nueva tarea para ejecutarse luego."
        # El formato de la tarea es:
        #          (tiempo_absoluto, tiempo
        self.tasks.append((self.time_counter + time_out, time_out, function, params))




import pilas

pilas.iniciar()
mono = pilas.actores.Mono()

def mover_al_mono(contexto):
    mono.x = contexto.x
    mono.y = contexto.y

pilas.eventos.mueve_mouse.connect(mover_al_mono)

pilas.avisar("Moviendo un actor en base a la posicion del mouse.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
import random
import unittest
import pilas

class MockEscena:

    def __init__(self):
        self.grupos = {1: [],
                       8: []}

class TestPiezas(unittest.TestCase):

    def test_pieza_de_ejemplo(self):

        filas = 3
        columnas = 3

        mock_escena = MockEscena()

        grilla = pilas.imagenes.Grilla("ejemplos/data/piezas.png", filas, columnas)
        p = pilas.ejemplos.piezas.Pieza(mock_escena, grilla, 1, filas, columnas)

        self.assertEqual(p.numero, 1)
        self.assertEqual(p.numero_derecha, 2)
        self.assertEqual(p.numero_izquierda, 0)
        self.assertTrue(p.numero_arriba < 0)
        self.assertEqual(p.numero_abajo, 4)

        p = pilas.ejemplos.piezas.Pieza(mock_escena, grilla, 8, filas, columnas)

        self.assertEqual(p.numero, 8)
        self.assertEqual(p.numero_derecha, -1)   # la pieza 8 no tiene borde derecho.
        self.assertEqual(p.numero_izquierda, 7) 
        self.assertEqual(p.numero_arriba, 5)
        self.assertEqual(p.numero_abajo, 11) # la pieza 8 no tiene parte de abajo

if __name__ == '__main__':
    pilas.iniciar()
    unittest.main()



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pilas
import random


class Piezas(pilas.escenas.Normal):
    """Representa la escena de rompecabezas.

    La escena comienza con una imagen que se descompone en muchos
    actores Pieza.
    """

    def __init__(self, ruta_a_la_imagen="ejemplos/data/piezas.png", filas=4, columnas=4):
        pilas.actores.utils.eliminar_a_todos()
        pilas.escenas.Normal.__init__(self, pilas.colores.grisoscuro)
        grilla = pilas.imagenes.Grilla(ruta_a_la_imagen, columnas, filas)
        self.crear_piezas(grilla, filas, columnas)
        self.pieza_en_movimiento = None

        pilas.eventos.click_de_mouse.conectar(self.al_hacer_click)
        pilas.eventos.termina_click.connect(self.al_soltar_el_click)
        pilas.eventos.mueve_mouse.connect(self.al_mover_el_mouse)

        self.sonido_tick = pilas.sonidos.cargar("tick.wav")

    def crear_piezas(self, grilla, filas, columnas):
        "Genera todas las piezas en base al tamaño del constructor."
        self.piezas = []
        self.grupos = {}

        for x in range(filas * columnas):
            self.grupos[x] = set([x])
            pieza = Pieza(self, grilla, x, filas, columnas)
            self.piezas.append(pieza)
            pieza.x = random.randint(-200, 200)
            pieza.y = random.randint(-200, 200)


    def al_hacer_click(self, evento):
        "Atiente cualquier click que realice el usuario en la pantalla."
        x, y = evento.x, evento.y
        pieza_debajo_de_mouse = pilas.actores.utils.obtener_actor_en(x, y)

        if pieza_debajo_de_mouse and isinstance(pieza_debajo_de_mouse, Pieza):
            self.pieza_en_movimiento = pieza_debajo_de_mouse
            self.pieza_en_movimiento.mostrar_arriba_todas_las_piezas()

    def al_soltar_el_click(self, evento):
        if self.pieza_en_movimiento:
            self.pieza_en_movimiento.soltar_todas_las_piezas_del_grupo()
            self.pieza_en_movimiento.mostrar_abajo_todas_las_piezas()
            self.pieza_en_movimiento = None

    def al_mover_el_mouse(self, evento):
        if self.pieza_en_movimiento:
            self.pieza_en_movimiento.x += evento.dx
            self.pieza_en_movimiento.y += evento.dy
            
    def conectar(self, pieza_a, pieza_b):
        a = pieza_a.numero
        b = pieza_b.numero

  
        if a in self.grupos[b]:
            #Evita contectar mas de una vez a dos piezas.
            return

        """Inicialmente comienzo con::


            0: [0, 1, 2]
            1: [0, 1, 2]
            2: [0, 1, 2]
            3: [3]

        ¿y si conecto la pieza 3 con la 2?

        - tendría que obtener todas las piezas que conoce 2.
        
        - iterar en ese grupo y decirle a cada pieza que sume a 3 en su grupo::

            0: [0, 1, 2, 3]
            1: [0, 1, 2, 3]
            2: [0, 1, 2, 3]

        - luego solo me falta tomar a uno de esos grupos actualizados
          y decirle a 3 que ese será su grupo::

            3: [0, 1, 2, 3]
        """
        
        grupo_nuevo = set(self.grupos[a]).union(self.grupos[b])

        for pieza in grupo_nuevo:
            self.grupos[pieza] = grupo_nuevo 

        self.sonido_tick.reproducir()
        
class Pieza(pilas.actores.Animado):
    """Representa una pieza del rompecabezas.

    Esta pieza se puede arrastrar con el mouse y cuando se suelta
    intentará conectarse con las demás."""

    def __init__(self, escena_padre, grilla, cuadro, filas, columnas):
        "Genera la pieza que representa una parte de la imagen completa."
        self.escena_padre = escena_padre
        self.numero = cuadro
        pilas.actores.Animado.__init__(self, grilla)

        self.z_de_la_pieza_mas_alta = 0
        self.asignar_numero_de_piezas_laterales(cuadro, columnas)

        self.definir_cuadro(cuadro)

        self.radio_de_colision = self.obtener_ancho() / 2 + 12
        self.piezas_conectadas = []

    def asignar_numero_de_piezas_laterales(self, cuadro, columnas):
        "Guarda el numero de las piezas que se pueden conectar en sus bordes."
        self.numero_arriba = cuadro - columnas
        self.numero_abajo = cuadro + columnas

        if cuadro % columnas == 0:
            self.numero_izquierda = -1
        else:
            self.numero_izquierda = cuadro - 1

        if cuadro % columnas == columnas -1:
            self.numero_derecha = -1
        else:
            self.numero_derecha = cuadro + 1

    def soltar_todas_las_piezas_del_grupo(self):
        for numero in self.escena_padre.grupos[self.numero]:
            pieza = self.escena_padre.piezas[numero]
            pieza.soltar()

    def soltar(self):
        # Busca todas las colisiones entre esta pieza
        # que se suelta y todas las demás.
        colisiones = pilas.colisiones.obtener_colisiones(self, self.escena_padre.piezas)

        for x in colisiones:
            self.intentar_conectarse_a(x)

    def se_pueden_conectar_los_bordes(self, borde1, borde2):
        distancia = pilas.utils.distancia(borde1, borde2)
        return  distancia < 12

    def intentar_conectarse_a(self, otra):
        "Intenta vincular dos piezas, siempre y cuando coincidan en sus bordes."

        # Intenta conectar los bordes laterales
        if self.numero_derecha == otra.numero:
            if self.se_pueden_conectar_los_bordes(self.derecha, otra.izquierda):
                otra.izquierda = self.derecha
                otra.arriba = self.arriba
                self.conectar_con(otra)

        elif self.numero_izquierda == otra.numero:
            if self.se_pueden_conectar_los_bordes(self.izquierda, otra.derecha):
                otra.derecha = self.izquierda
                otra.arriba = self.arriba
                self.conectar_con(otra)

        # Intenta conectar los bordes superior e inferior
        if self.numero_abajo == otra.numero:
            if self.se_pueden_conectar_los_bordes(self.abajo, otra.arriba):
                otra.arriba = self.abajo
                otra.izquierda = self.izquierda
                self.conectar_con(otra)

        elif self.numero_arriba == otra.numero:
            if self.se_pueden_conectar_los_bordes(self.arriba, otra.abajo):
                otra.abajo = self.arriba
                otra.izquierda = self.izquierda
                self.conectar_con(otra)


    def conectar_con(self, otra_pieza):
        self.escena_padre.conectar(self, otra_pieza)


    def __repr__(self):
        return "<<Pieza %d>>" %(self.animacion.obtener_cuadro())


    def set_x(self, x):
        "A diferencia de los actores normales, las piezas tienen que mover a todo su grupo."
        dx = x - self.x

        for numero in self.escena_padre.grupos[self.numero]:
            try:
                pieza = self.escena_padre.piezas[numero]
                pieza.definir_posicion(pieza.x + dx, pieza.y)
            except IndexError:
                pass

    def set_y(self, y):
        "A diferencia de los actores normales, las piezas tienen que mover a todo su grupo."
        dy = y - self.y

        for numero in self.escena_padre.grupos[self.numero]:
            try:
                pieza = self.escena_padre.piezas[numero]
                pieza.definir_posicion(pieza.x, pieza.y + dy)
            except IndexError:
                pass

    def get_x(self):
        x, y = self.obtener_posicion()
        return x

    def get_y(self):
        x, y = self.obtener_posicion()
        return y

    x = property(get_x, set_x, doc="Define la posición horizontal.")
    y = property(get_y, set_y, doc="Define la posición vertical.")

    def mostrar_arriba_todas_las_piezas(self):
        for numero in self.escena_padre.grupos[self.numero]:
            pieza = self.escena_padre.piezas[numero]
            pieza.z = -1

    def mostrar_abajo_todas_las_piezas(self):
        for numero in self.escena_padre.grupos[self.numero]:
            pieza = self.escena_padre.piezas[numero]
            pieza.z = 0



import pilas
import sys

pilas.iniciar()

mono = pilas.actores.Mono()

bomba = pilas.actores.Bomba()
bomba.x = 200

banana = pilas.actores.Banana()
banana.x = -200

pilas.avisar("Mostrando tres actores de ejemplo.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Texto

class Opcion(Texto):

    def __init__(self, texto, x=0, y=0, funcion_a_invocar=None):
        Texto.__init__(self, texto, x=x, y=y)
        self.funcion_a_invocar = funcion_a_invocar

    def resaltar(self, estado=True):
        "Pinta la opcion actual de un color mas claro."

        if estado:
            self.color = pilas.colores.blanco
        else:
            self.color = pilas.colores.negro

    def seleccionar(self):
        "Invoca a la funcion que tiene asociada para ejecutar."

        if self.funcion_a_invocar:
            self.funcion_a_invocar()
        else:
            print "Cuidado, la opcion", self, "no tiene funcion asociada."



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import time
import sys

import ventana
import control
import camara
import escenas
import utils
import eventos
import tareas
import pytweener
import pilas
import modos_de_mundo


class Mundo:
    """Representa un objeto unico que mantiene en funcionamiento al motor.

    Mundo tiene como responsabilidad iniciar los componentes del
    motor y mantener el bucle de juego.

    Este objeto delega en otros el modo de ejecucion en un momento dado, 
    por ejemplo cuando inicia se usa el "ModoEjecucionNormal" y cuando el usuario
    pulsa F12 este modo cambia por "ModoEjecucionNormal".
    """

    def __init__(self, ancho, alto, titulo):
        self.ventana = ventana.iniciar(ancho, alto, titulo)
        ventana.ancho = ancho
        ventana.alto = alto

        self.control = control.Control()

        # todo: llevar a ventana.iniciar
        utils.hacer_flotante_la_ventana()
        pilas.motor.centrar_ventana()

        self.camara = camara.Camara(self.ventana)

        self.escena_actual = None

        # Genera los administradores de tareas e interpolaciones.
        self.tweener = pytweener.Tweener()
        self.tasks = tareas.Tareas() 

        self.modo_ejecucion = None
        self.definir_modo_ejecucion(modos_de_mundo.ModoEjecucionNormal(self))
        self.salir = False

    def definir_modo_ejecucion(self, nuevo_modo):
        if self.modo_ejecucion:
            self.modo_ejecucion.salir()

        self.modo_ejecucion = nuevo_modo

    def terminar(self):
        self.salir = True

    def ejecutar_bucle_principal(self, ignorar_errores=False):
        "Mantiene en funcionamiento el motor completo."

        fps = pilas.fps.FPS(60)

        while not self.salir:

            # Invoca varias veces a la actualizacion si el equipo
            # es lento.
            for x in range(fps.actualizar()):
                # Mantiene el control de tiempo y lo reporta al sistema
                # de interpolaciones y tareas.
                self._realizar_actualizacion_logica(ignorar_errores)

            self._realizar_actualizacion_grafica()

        self._cerrar_ventana()

    def _realizar_actualizacion_logica(self, ignorar_errores):
        self.modo_ejecucion.actualizar_simuladores()

        # Emite el aviso de actualizacion a los receptores.
        self.modo_ejecucion.emitir_evento_actualizar()

        # Procesa todos los eventos.
        pilas.motor.procesar_y_emitir_eventos()

        # Analiza colisiones entre los actores
        if ignorar_errores:
            try:
                self.modo_ejecucion.analizar_colisiones()
            except Exception, e:
                print e
        else:
            self.modo_ejecucion.analizar_colisiones()

        # Dibuja la escena actual y a los actores
        if ignorar_errores:
            try:
                self.escena_actual.actualizar()
            except Exception, e:
                print e
        else:
            self.escena_actual.actualizar()
        
        if ignorar_errores:
            try:
                self.modo_ejecucion.actualizar_actores()
            except Exception, e:
                print e
        else:
            self.modo_ejecucion.actualizar_actores()

    def _realizar_actualizacion_grafica(self):
        self.escena_actual.dibujar(self.ventana)
        self.modo_ejecucion.dibujar_actores()

        pilas.motor.actualizar_pantalla()

    def _cerrar_ventana(self):
        pilas.motor.cerrar_ventana()
        sys.exit(0)

    def definir_escena(self, escena_nueva):
        "Cambia la escena que se muestra en pantalla"

        if self.escena_actual:
            eliminar_actores = True
        else:
            eliminar_actores = False

        self.escena_actual = escena_nueva


    def agregar_tarea(self, time_out, function, *params): 
        self.tasks.agregar(time_out, function, params)

    def alternar_pausa(self):
        self.modo_ejecucion.alternar_pausa()

    def alternar_modo_depuracion(self):
        self.modo_ejecucion.alternar_modo_depuracion()



"""
This file is part of the 'Elements' Project
Elements is a 2D Physics API for Python (supporting Box2D2)

Copyright (C) 2008, The Elements Team, <elements@linuxuser.at>

Home:  http://elements.linuxuser.at
IRC:   #elements on irc.freenode.org

Code:  http://www.assembla.com/wiki/show/elements
       svn co http://svn2.assembla.com/svn/elements                     

License:  GPLv3 | See LICENSE for the full text
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.              
"""
from math import pi
from math import cos
from math import sin
from math import sqrt

import tools

# Functions of a rendering class 
# mandatory:
#    __init__
#    start_drawing
#    after_drawing
#    draw_circle
#    draw_polygon
#    draw_lines
#    set_lineWidth
#
# renderer-specific mandatory functions:
# for pygame:
#    set_surface
# for cairo:
#    draw_text
# for opengl:
#    

# IMPORTANT
# The drawing functions get the coordinates in their screen coordinate system
# no need for translations anymore :)

class draw_pygame(object):
    """ This class handles the drawing with pygame, which is really
        simple since we only need draw_ellipse and draw_polygon.
    """
    lineWidth = 0
    
    def __init__(self):
        """ Load pygame.draw and pygame.Rect, and reference it for
            the drawing methods
            
            Parameters:
              surface .... pygame surface (default: None)
              lineWidth .. 

            Return: Class draw_pygame()
        """
        print "* Pygame selected as renderer"        
        from pygame import draw
        from pygame import Rect
        
        self.draw = draw
        self.Rect = Rect

    def set_lineWidth(self, lw):
        """
        """
        self.lineWidth = lw

    def set_surface(self, surface):
        """
        """
        self.surface = surface

    def get_surface(self):
        """
        """
        return self.surface

    def start_drawing(self):
        pass
        
    def after_drawing(self):
        pass

    def draw_circle(self, clr, pt, radius, angle):
        """ Draw a circle
        
            Parameters:
              pt ........ (x, y)
              clr ....... color in rgb ((r), (g), (b))
              radius .... circle radius 
              angle ..... rotation in radians
              
            Return: -
        """
        x, y = pt
        
        x1 = x - radius
        y1 = y - radius
        
        rect = self.Rect( [x1, y1, 2*radius, 2*radius] )
        self.draw.ellipse(self.surface, clr, rect, self.lineWidth)
                        
        # draw the orientation vector
        if radius > 10:
            rx = cos(angle) * radius
            ry = -sin(angle) * radius
            
            self.draw.line(self.surface, (255,255,255), pt, (x+rx, y+ry))

    def draw_polygon(self, clr, points):
        """ Draw a polygon
        
            Parameters:
              clr ....... color in rgb ((r), (g), (b))
              points .... polygon points in normal (x,y) positions
              
            Return: -
        """
        self.draw.polygon(self.surface, clr, points, self.lineWidth)
        #self.draw.lines(self.surface, clr, True, points)

    def draw_lines(self, clr, closed, points, width=None):
        """ Draw a polygon
        
            Parameters:
              clr ....... color in rgb ((r), (g), (b))
              points .... polygon points in normal (x,y) positions
              
            Return: -
        """
        if width == None:
            lw = self.lineWidth
        else: 
            lw = width
            
        self.draw.lines(self.surface, clr, closed, points, lw)

class draw_cairo(object):
    """ This class handles the drawing with cairo, which is really
        simple since we only need draw_ellipse and draw_polygon.
    """
    window = None
    da     = None
    circle_surface = None
    box_surface    = None

    def __init__(self, drawMethod="filled"):
        """ Load cairo.draw and cairo.Rect, and reference it for
            the drawing methods
            
            Return: Class draw_cairo()
        """
        print "* Cairo selected as renderer"
        import cairo
        self.cairo = cairo
        self.set_drawing_method(drawMethod)
        #self.draw_box = self.draw_box_image

    def set_lineWidth(self, lw): # unused
        self.lineWidth = lw 

    def set_pizarra(self, pizarra):
        self.pizarra = pizarra

    def set_drawing_area(self, da):
        """ Set the area for Cairo to draw to
        
            da ...... drawing area (gtk.DrawingArea)

            Return: -
        """
        self.da = da
        self.window = da.window
        print "* Cairo renderer drawing area set"

    def set_drawing_method(self, type):
        """ type = filled, image """
        self.draw_circle = getattr(self, "draw_circle_%s" % type)
        #self.draw_box    = getattr(self, "draw_box_%s" % type)

    def start_drawing(self):
        #self.width, self.height = self.window.get_size()
        self.imagesurface = self.cairo.ImageSurface(self.cairo.FORMAT_ARGB32, 640, 480)
        self.ctx = ctx = self.cairo.Context(self.imagesurface)

        ctx.set_source_rgb(1, 1, 1) # background color
        ctx.paint()

        ctx.move_to(0, 0)
        ctx.set_source_rgb(0, 0, 0) # defaults for the rest of the drawing
        ctx.set_line_width(1)
        ctx.set_tolerance(0.1)

        ctx.set_line_join(self.cairo.LINE_CAP_BUTT)
        # LINE_CAP_BUTT, LINE_CAP_ROUND, LINE_CAP_SQUARE, LINE_JOIN_BEVEL, LINE_JOIN_MITER, LINE_JOIN_ROUND
        
        #ctx.set_dash([20/4.0, 20/4.0], 0)

    def after_drawing(self):
        dest_ctx = self.pizarra.canvas.context
        dest_ctx.set_source_surface(self.imagesurface)
        dest_ctx.paint()

    def set_circle_image(self, filename):
        self.circle_surface = self.cairo.ImageSurface.create_from_png(filename)
        self.draw_circle = self.draw_circle_image

#    def set_box_image(self, filename):
#        self.box_surface = self.cairo.ImageSurface.create_from_png(filename)
#        self.draw_box = self.draw_box_image

    def draw_circle_filled(self, clr, pt, radius, angle=0):
        x, y = pt

        clr = tools.rgb2floats(clr)
        self.ctx.set_source_rgb(*clr)
        self.ctx.move_to(x, y)
        self.ctx.arc(x, y, radius, 0, 2*3.1415)
        self.ctx.fill()

    def draw_circle():
        pass

    def draw_circle_image(self, clr, pt, radius, angle=0, sf=None):
        if sf == None:
            sf = self.circle_surface
        x, y = pt
        self.ctx.save()
        self.ctx.translate(x, y)
        self.ctx.rotate(-angle)
        image_r = sf.get_width() / 2
        scale = float(radius) / image_r
        self.ctx.scale(scale, scale)
        self.ctx.translate(-0.5*sf.get_width(), -0.5*sf.get_height())
        self.ctx.set_source_surface(sf)
        self.ctx.paint()
        self.ctx.restore()

    def draw_image(self, source, pt, scale=1.0, rot=0, sourcepos=(0,0)):
        self.ctx.save()
        self.ctx.rotate(rot)
        self.ctx.scale(scale, scale)
        destx, desty = self.ctx.device_to_user_distance(pt[0], pt[1])
        self.ctx.set_source_surface(source, destx-sourcepos[0], desty-sourcepos[1])
        self.ctx.rectangle(destx, desty, source.get_width(), source.get_height())
        self.ctx.fill()
        self.ctx.restore()
                 
    def draw_polygon(self, clr, points):
        """ Draw a polygon
        
            Parameters:
              clr ....... color in rgb ((r), (g), (b))
              points .... polygon points in normal (x,y) positions
              
            Return: -
        """        
        clr = tools.rgb2floats(clr)
        self.ctx.set_source_rgb(clr[0], clr[1], clr[2])

        pt = points[0]
        self.ctx.move_to(pt[0], pt[1])
        for pt in points[1:]:
            self.ctx.line_to(pt[0], pt[1])
            
        self.ctx.fill()
   
    def draw_text(self, text, center, clr=(0,0,0), size=12, fontname="Georgia"):
        clr = tools.rgb2floats(clr)
        self.ctx.set_source_rgb(clr[0], clr[1], clr[2])

        self.ctx.select_font_face(fontname, self.cairo.FONT_SLANT_NORMAL, self.cairo.FONT_WEIGHT_NORMAL)
        self.ctx.set_font_size(size)
        x_bearing, y_bearing, width, height = self.ctx.text_extents(text)[:4]
        self.ctx.move_to(center[0] + 0.5 - width / 2 - x_bearing, center[1] + 0.5 - height / 2 - y_bearing)
        self.ctx.show_text(text)

    def draw_lines(self, clr, closed, points):
        """ Draw a polygon
        
            Parameters:
              clr ....... color in rgb ((r), (g), (b))
              closed .... whether or not to close the lines (as a polygon)
              points .... polygon points in normal (x,y) positions
            Return: -
        """        
        clr = tools.rgb2floats(clr)
        self.ctx.set_source_rgb(clr[0], clr[1], clr[2])

        pt = points[0]
        self.ctx.move_to(pt[0], pt[1])
        for pt in points[1:]:
            self.ctx.line_to(pt[0], pt[1])

        if closed:
            pt = points[0]
            self.ctx.line_to(pt[0], pt[1])

        self.ctx.stroke()

class draw_opengl_pyglet(object):
    """ This class handles the drawing with pyglet
    """
    lineWidth = 0
    def __init__(self):
        """ Load pyglet.gl, and reference it for the drawing methods
            
            Parameters:
              surface .... not used with pyglet
              lineWidth .. 
        """
        print "* OpenGL_Pyglet selected as renderer"

        from pyglet import gl
        self.gl = gl

    def set_lineWidth(self, lw):
        self.lineWidth = lw
    
    def draw_circle(self, clr, pt, radius, a=0):
        clr = tools.rgb2floats(clr)
    	self.gl.glColor3f(clr[0], clr[1], clr[2])

        x, y = pt
    	segs = 15
    	coef = 2.0*pi/segs;
    	
    	self.gl.glBegin(self.gl.GL_LINE_LOOP)
    	for n in range(segs):
    		rads = n*coef
    		self.gl.glVertex2f(radius*cos(rads + a) + x, radius*sin(rads + a) + y)
    	self.gl.glVertex2f(x,y)
    	self.gl.glEnd()

    def draw_polygon(self, clr, points):
        clr = tools.rgb2floats(clr)
    	self.gl.glColor3f(clr[0], clr[1], clr[2])
    	
        self.gl.glBegin(self.gl.GL_LINES)

        p1 = points[0]
        for p in points[1:]:
            x1, y1 = p1
            x2, y2 = p1 = p
            
            self.gl.glVertex2f(x1, y1)
            self.gl.glVertex2f(x2, y2)

        x1, y1 = points[0]
        
        self.gl.glVertex2f(x2, y2)
        self.gl.glVertex2f(x1, y1)

        self.gl.glEnd()
                
    def draw_lines(self, clr, closed, points):
        pass

    def start_drawing(self):
        pass
        
    def after_drawing(self):
        pass



#!/usr/bin/env python
from setuptools import setup
from setuptools import find_packages


setup(
        name='pilas',
        version='0.38',
        description='A simple to use video game framework.',
        author='Hugo Ruscitti',
        author_email='hugoruscitti@gmail.com',
        install_requires=[
            'setuptools',
            'pymunk',
            'Box2D',
            ],
        packages=['pilas', 'pilas.elements', 'pilas.actores', 'pilas.dispatch', 'pilas.ejemplos', 'pilas.motores'],
        url='http://www.pilas-engine.com.ar',

        include_package_data = True,
        package_data = {
            'images': ['pilas/data/*', 'pilas/ejemplos/data/*', 'pilas/data/fondos/*',
                       'pilas/data/juegobase/*', 'pilas/data/juegobase/data/*'],
            },

        scripts=['bin/pilas'],

        classifiers = [
            'Intended Audience :: Developers',
            'Intended Audience :: Education',
            'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
            'Natural Language :: Spanish',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Topic :: Games/Entertainment',
            'Topic :: Education',
            'Topic :: Software Development :: Libraries',
          ],
    )




import pilas
import sys

pilas.iniciar()
pizarra = pilas.actores.Pizarra()

def dibujar_en_la_pizarra(evento):
    pizarra.pintar_punto(evento.x, evento.y)

pilas.eventos.mueve_mouse.connect(dibujar_en_la_pizarra)

pilas.avisar("Usa el mouse para dibujar.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Animacion
import math

class Disparo(Animacion):
    "Representa un disparo que avanza."

    def __init__(self, x=0, y=0, rotacion=0, velocidad=2):
        self.velocidad = velocidad
        grilla = pilas.imagenes.Grilla("disparo.png", 2)
        Animacion.__init__(self, grilla, ciclica=True, x=x, y=y)
        self.radio_de_colision = 10
        self.rotacion = rotacion

    def actualizar(self):
        Animacion.actualizar(self)
        self.avanzar()

    def avanzar(self):
        "Hace avanzar la nave en direccion a su angulo."
        rotacion_en_radianes = math.radians(-self.rotacion + 90)
        dx = math.cos(rotacion_en_radianes) * self.velocidad
        dy = math.sin(rotacion_en_radianes) * self.velocidad
        self.x += dx
        self.y += dy



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pilas

class Fondo(pilas.actores.Actor):

    def __init__(self, imagen):
        pilas.actores.Actor.__init__(self, imagen)
        self.z = 100


class Volley(Fondo):
    "Muestra una escena que tiene un fondo de pantalla de paisaje."

    def __init__(self):
        Fondo.__init__(self, "fondos/volley.png")

class Pasto(Fondo):
    "Muestra una escena que tiene un fondo de pantalla de paisaje."

    def __init__(self):
        Fondo.__init__(self, "fondos/pasto.png")

class Selva(Fondo):
    "Muestra una escena que tiene un fondo de pantalla de paisaje."

    def __init__(self):
        Fondo.__init__(self, "fondos/selva.png")


class Tarde(Fondo):
    "Representa una escena de fondo casi naranja."

    def __init__(self):
        Fondo.__init__(self, "fondos/tarde.png")


class Espacio(Fondo):
    "Es un espacio con estrellas."

    def __init__(self):
        Fondo.__init__(self, "fondos/espacio.png")




# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Animado
import copy

VELOCIDAD = 4


class Animacion(Animado):
    """Representa una animacion de una grilla de imagenes.

    Este actor toma una grilla de cuadros de animacion
    y los reproduce hasta que la animacion termina. Cuando
    la animacion termina se elimina a si mismo.
    """

    def __init__(self, grilla, ciclica=False, x=0, y=0):
        Animado.__init__(self, grilla, x=x, y=y)
        self.tick = 0
        self.ciclica = ciclica     # Indica si la animacion debe reiniciar luego de terminar.

    def actualizar(self):
        if self.tick > VELOCIDAD:
            self.tick = 0
            ha_reiniciado = self.animacion.avanzar()
            self.animacion.asignar(self)

            # Si la animacion ha terminado se elimina de la pantalla.
            if ha_reiniciado and not self.ciclica:
                self.eliminar()
        else:
            self.tick += 1



import pilas
pilas.iniciar()

# ejemplo de uso boton personalizado
# al iniciar no se visualiza nada pero al cliquear al centro
# aparece de pronto un circulo, has la prueba !


# pilas.actores.Boton(x, y, "imagen normal.png, imagen press.png, imagen over.png")

boton = pilas.actores.Boton(0, 0,'ficha_vacia.png', 'ficha1.png', 'ficha1.png')

# al cargar previamente nuevas imagenes
# cuando llamemos sus metodos como:
# boton.pintar_presionado() se cambiara por la imagen
# cargada en la parte de imagen_press.png

# como cargamos ficha_vacia.png en la seccion de imagen normal
# la cara principal del boton seria invisible

class contador:
    presionado_una_vez = True

def presionar():
    if (contador.presionado_una_vez == True):
        # pintamos boton seccion imagen_press
        boton.pintar_presionado()
        boton.escala = 0.8
        boton.escala = pilas.interpolar([1], duracion=0.1)
        contador.presionado_una_vez = False
        pilas.avisar("Epa!!")
        

#conectamos funcion a evento presionar boton
boton.conectar_presionado(presionar) 

pilas.avisar("Pulsa exactamente en el centro de la ventana.")

pilas.ejecutar()


# si no queremos un boton personalizado simplemente

# escribimos boton = pilas.actores.Boton()

# y le conectamos sus metodos:

# boton.conectar_presionado(boton.pintar_presionado)
# boton.conectar_normal(boton.pintar_normal)

# ...

# o podemos conectar de la forma

# def pintar_cuando_este_sobre():
#     boton.pintar_sobre()

# boton.conectar_presionado(pintar_cuando_este_sobre())


















import pilas

pilas.iniciar()
nave = pilas.actores.Nave()


piedras = pilas.atajos.fabricar(pilas.actores.Piedra, 10, tamano='media')
piedras.aprender(pilas.habilidades.PuedeExplotar)
piedras.aprender(pilas.habilidades.RebotaComoPelota)

nave.definir_enemigos(piedras)

pilas.atajos.definir_gravedad(1, 1)




pilas.ejecutar()



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pygame
from pilas.simbolos import *
import cairo
import pilas
from pilas import eventos

ANCHO = 640
ALTO = 480

class Pygame:
    """Representa la capa de interaccion con la biblioteca Pygame.

    Esto permite que tus juegos realizados con pilas funcionen
    en cualquier equipo que tenga instalada la biblioteca pygame,
    por ejemplo equipos como OLPC o simplemente aquellos que no
    tengan aceleradoras de graficos OpenGL."""

    import pilas.baseactor

    class Color(pygame.Color):

        def obtener_componentes(self):
            return (self.r, self.g, self.b)

    class Canvas:
        "Representa una superficie sobre la que se puede dibujar usando cairo."

        def __init__(self):
            import array
            self.data = array.array('c', chr(0) * ANCHO * ALTO * 4)
            stride = ANCHO * 4
            self.surface = cairo.ImageSurface.create_for_data(self.data, cairo.FORMAT_ARGB32, 
                    ANCHO, ALTO, stride)

            self.context = cairo.Context(self.surface)
            self.actualizar()

        def actualizar(self):
            #self.image.LoadFromPixels(ANCHO, ALTO, self.surface.get_data())
            self.image = pygame.image.frombuffer(self.data.tostring(), (ANCHO, ALTO), "RGBA")

    class Grilla:
        """Representa una grilla de imagenes con varios cuadros de animación.

        Una grilla es un objeto que se tiene que inicializar con la ruta
        a una imagen, la cantidad de columnas y filas.

        Por ejemplo, si tenemos una grilla con 2 columnas y 3 filas
        podemos asociarla a un actor de la siguiente manera::

            grilla = pilas.imagenes.Grilla("animacion.png", 2, 3)
            grilla.asignar(actor)

        Entonces, a partir de ahora nuestro actor muestra solamente un
        cuadro de toda la grilla.

        Si quieres avanzar la animacion tienes que modificar el objeto
        grilla y asignarlo nuevamente al actor::

            grilla.avanzar()
            grilla.asignar(actor)
        """

        def __init__(self, ruta, columnas=1, filas=1):
            self.image = pilas.imagenes.cargar(ruta)
            self.cantidad_de_cuadros = columnas * filas
            self.columnas = columnas
            self.filas = filas
            self.cuadro_ancho = self.image.get_width() / columnas
            self.cuadro_alto = self.image.get_height() / filas
            self.crear_grilla_de_imagenes()
            self.definir_cuadro(0)

        def crear_grilla_de_imagenes(self):
            "Genera una lista con los cuadros de animación en una grilla"
                                
            tile_w = self.cuadro_ancho
            tile_h = self.cuadro_alto
            self.imagenes = []

            for c in range(self.columnas):
                for r in range(self.filas):
                    rect = pygame.Rect(c * (tile_w) , r * (tile_h), tile_w, tile_h)
                    self.imagenes.append(self.image.subsurface(rect).copy())


        def definir_cuadro(self, cuadro):
            self.cuadro = cuadro

            '''
            frame_col = cuadro % self.columnas
            frame_row = cuadro / self.columnas

            dx = frame_col * self.cuadro_ancho - self.sub_rect.Left
            dy = frame_row * self.cuadro_alto - self.sub_rect.Top
            '''

            #self.sub_rect.Offset(dx, dy)

        def asignar(self, sprite):
            "Sets the sprite's image with animation state."

            cuadro_de_imagen = self.imagenes[self.cuadro]
            sprite.definir_imagen(cuadro_de_imagen)
            sprite.rect.width = cuadro_de_imagen.get_width()
            sprite.rect.height = cuadro_de_imagen.get_height()

        def avanzar(self):
            ha_reiniciado = False
            cuadro_actual = self.cuadro + 1

            if cuadro_actual >= self.cantidad_de_cuadros:
                cuadro_actual = 0
                ha_reiniciado = True

            self.definir_cuadro(cuadro_actual)
            return ha_reiniciado

        def obtener_cuadro(self):
            return self.cuadro

    class Sonido:

        def __init__(self, ruta):
            self.sonido = pygame.mixer.Sound(ruta)

        def reproducir(self):
            self.sonido.play()
        
        def Play(self):
            self.reproducir()

    class Actor(pilas.baseactor.BaseActor, pygame.sprite.Sprite):
        """Representa un objeto visible en pantalla, algo que se ve y tiene posicion.

        Un objeto Actor se tiene que crear siempre indicando la imagen, ya
        sea como una ruta a un archivo como con un objeto Image. Por ejemplo::

            protagonista = Actor("protagonista_de_frente.png")

        es equivalente a:

            imagen = pilas.imagenes.cargar("protagonista_de_frente.png")
            protagonista = Actor(imagen)

        Luego, na vez que ha sido ejecutada la sentencia aparecerá en el centro de
        la pantalla el nuevo actor para que pueda manipularlo. Por ejemplo
        alterando sus propiedades::

            protagonista.x = 100
            protagonista.scale = 2
            protagonista.rotation = 30


        Estas propiedades tambien se pueden manipular mediante
        interpolaciones. Por ejemplo, para aumentar el tamaño del
        personaje de 1 a 5 en 7 segundos::

            protagonista.scale = pilas.interpolar(1, 5, 7)

        Si creas un sprite sin indicar la imagen se cargará
        una por defecto.
        """

        def __init__(self, image="sin_imagen.png", x=0, y=0):
            self.centro_x = 0
            self.centro_y = 0

            if isinstance(image, str):
                image = pilas.imagenes.cargar(image)

            pygame.sprite.Sprite.__init__(self)
            self.image = image
            self.image_original = image.convert_alpha()
            self.rect = self.image.get_rect()
            pilas.baseactor.BaseActor.__init__(self, x=x, y=y)
            self._escala_actual = 1
            self._rotacion_actual = 0
            
        def definir_imagen(self, imagen):
            self.image = imagen

            if self._rotacion_actual:
                self.rotacion = self._rotacion_actual

        def obtener_imagen(self):
            return self.image

        def dibujar(self, aplicacion):
            x, y = self.rect.topleft
            aplicacion.blit(self.image, (x + 320 - self.centro_x - pilas.motor.camara_x, 
                    y + 240 - self.centro_y - pilas.motor.camara_y))

        def duplicar(self, **kv):
            duplicado = self.__class__()

            for clave in kv:
                setattr(duplicado, clave, kv[clave])

            return duplicado

        def obtener_ancho(self):
            return self.rect.width

        def obtener_alto(self):
            return self.rect.height

        def __str__(self):
            return "<%s en (%d, %d)>" %(self.__class__.__name__, self.x, self.y)

        def obtener_area(self):
            return self.obtener_ancho(), self.obtener_alto()

        def definir_centro(self, x, y):
            self.centro_x = x
            self.centro_y = y

        def obtener_posicion(self):
            x, y = self.rect.topleft
            return x, -y

        def definir_posicion(self, x, y):
            self.rect.topleft = (x, -y)

        def obtener_escala(self):
            return self._escala_actual

        def definir_escala(self, s):
            self.image = pygame.transform.rotozoom(self.image, 0, s)
            self._escala_actual = s

        def obtener_rotacion(self):
            return self._rotacion_actual

        def definir_rotacion(self, r):
            self.image = pygame.transform.rotozoom(self.image_original, -r, 1)
            self._rotacion_actual = r

            # Se asegura que el personaje gire desde el centro
            # del area de colision.
            centro = self.rect.center
            self.rect = self.image.get_rect()
            self.rect.center = centro

    class Texto(pilas.baseactor.BaseActor, pygame.sprite.Sprite):
        """Representa un texto en pantalla.

        El texto tiene atributos como ``texto``, ``magnitud`` y ``color``.
        """

        def __init__(self, texto="None", x=0, y=0):
            self.centro_x = 0
            self.centro_y = 0
            pygame.font.init()
            self.font = pygame.font.Font(None, 32)
            self._color = pilas.colores.negro

            pygame.sprite.Sprite.__init__(self)
            self.set_text(texto)
            self.rect = self.image.get_rect()
            pilas.baseactor.BaseActor.__init__(self, x=x, y=y) 
            self._escala_actual = 1
            self._texto = texto

        def get_text(self):
            return self._texto

        def set_text(self, texto):
            self._texto = texto
            self.image = self.font.render(texto, 1, self.color.obtener_componentes())

        def get_size(self):
            return 32

        def set_size(self, size):
            print "No se puede cambiar el tamanano del texto en pygame"

        '''
        def _set_central_axis(self):
            rect = self.GetRect()
            size = (rect.GetWidth(), rect.GetHeight())
            self.SetCenter(size[0]/2, size[1]/2)
        '''

        def obtener_posicion(self):
            x, y = self.rect.topleft
            return x, y

        def definir_posicion(self, x, y):
            self.rect.topleft = (x, -y)

        def get_color(self):
            return self._color

        def set_color(self, k):
            self._color = k
            self.texto = self._texto

        texto = property(get_text, set_text, doc="El texto que se tiene que mostrar.")
        magnitud = property(get_size, set_size, doc="El tamaño del texto.")
        color = property(get_color, set_color, doc="Color del texto.")

        def dibujar(self, aplicacion):
            x, y = self.rect.topleft
            #aplicacion.blit(self.image, (x + 320 - self.centro_x, y + 240 - self.centro_y))
            aplicacion.blit(self.image, (x + 320 - self.centro_x - pilas.motor.camara_x, 
                    y + 240 - self.centro_y - pilas.motor.camara_y))

        def colisiona_con_un_punto(self, x, y):
            return False

        def obtener_ancho(self):
            return self.rect.width

        def obtener_alto(self):
            return self.rect.height

        def obtener_area(self):
            return self.obtener_ancho(), self.obtener_alto()

        def definir_centro(self, x, y):
            self.centro_x = x
            self.centro_y = y


    def __init__(self):
        pygame.init()

        if pygame.image.get_extended() != 1:
            print "Cuidado, en este equipo pygame no soporta archivos PNG. Instale SDL_image para resolver este problema."

        self.camara_x = 0
        self.camara_y = 0

    def crear_ventana(self, ancho, alto, titulo):
        self.ventana = pygame.display.set_mode((ancho, alto))
        pygame.display.set_caption(titulo)
        return self.ventana

    def cerrar_ventana(self):
        pygame.display.quit()

    def dibujar_circulo(self, x, y, radio, color, color_borde):
        pygame.draw.circle(self.ventana, (200, 0, 0), (x + 320 - pilas.motor.camara_x, 
            240 - y - pilas.motor.camara_y), radio/2, True)
        #aplicacion.blit(self.image, (x + 320 - self.centro_x - pilas.motor.camara_x, 
        #        y + 240 - self.centro_y - pilas.motor.camara_y))


    def cargar_sonido(self, ruta):
        return Pygame.Sonido(ruta)

    def centrar_ventana(self):
        pass

    def pulsa_tecla(self, tecla):
        "Indica si una tecla esta siendo pulsada en este instante."

        mapa = {
                IZQUIERDA: pygame.K_LEFT,
                DERECHA: pygame.K_RIGHT,
                ARRIBA: pygame.K_UP,
                ABAJO: pygame.K_DOWN,
                BOTON: pygame.K_SPACE,
                SELECCION: pygame.K_RETURN,
                }

        return pygame.key.get_pressed()[mapa[tecla]]

    def procesar_y_emitir_eventos(self):
        "Procesa todos los eventos que la biblioteca pygame pone en una cola."

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pilas.mundo.terminar()
            elif event.type == pygame.KEYDOWN:
                self.procesar_evento_teclado(event)

                if event.key == pygame.K_q:
                    pilas.mundo.terminar()
            elif event.type == pygame.MOUSEMOTION:
                # Notifica el movimiento del mouse con una señal
                x, y = event.pos
                dx, dy = event.rel
                y = -y
                dy = -dy

                x -= 320
                y += 240

                eventos.mueve_mouse.send("ejecutar", x=x, y=y, dx=dx, dy=dy)

            elif event.type == pygame.MOUSEBUTTONDOWN:
                x, y = event.pos
                x -= 320
                y -= 240

                if event.button in (4, 5):
                    if event.button == 4:
                        eventos.mueve_rueda.send("ejecutar", delta=1)
                    else:
                        eventos.mueve_rueda.send("ejecutar", delta=-1)
                else:
                    eventos.click_de_mouse.send("ejecutar", button=event.button, x=x, y=-y)

            elif event.type == pygame.MOUSEBUTTONUP:
                x, y = event.pos
                x -= 320
                y -= 240

                eventos.termina_click.send("ejecutar", button=event.button, x=x, y=-y)

    def procesar_evento_teclado(self, event):
        eventos.pulsa_tecla.send("ejecutar", code=event.key)

        if event.key == pygame.K_p:
            pilas.mundo.alternar_pausa()
        elif event.key == pygame.K_F12:
            pilas.mundo.alternar_modo_depuracion()
        elif event.key == pygame.K_ESCAPE:
            eventos.pulsa_tecla_escape.send("ejecutar")


    def actualizar_pantalla(self):
        pygame.display.flip()

    def ocultar_puntero_del_mouse(self):
        pygame.mouse.set_visible(True)

    def mostrar_puntero_del_mouse(self):
        pygame.mouse.set_visible(False)

    def pintar(self, color):
        self.ventana.fill(color)

    def cargar_imagen(self, ruta):
        # TODO: Optimizar la imagen preservando el canal alpha.
        return pygame.image.load(ruta)

    def obtener_centro_de_la_camara(self):
        return self.camara_x, self.camara_y

    def definir_centro_de_la_camara(self, x, y):
        self.camara_x, self.camara_y = x, y


    def generar_imagen_cairo(self, imagen):
        """Retorna una superficie de cairo representando a la imagen.

        Esta funcion es util para pintar imagenes sobre una pizarra
        o el escenario de un videojuego.
        """
        #import array

        pixels = imagen.get_buffer()
        w = imagen.get_width()
        h = imagen.get_height()

        return cairo.ImageSurface.create_for_data(pixels, cairo.FORMAT_RGB24, w, h)



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Animacion
from pilas.actores import Explosion

class Bomba(Animacion):
    """Representa una explosion para una bomba, dinamita etc...

    La bomba adquiere la habilidad explotar al momento de crearse, así
    que puedes invocar a su método "explotar" y la bomba hará un
    explosión con sonido en pantalla.

    Este es un ejemplo de uso del actor::

        bomba = pilas.actores.Bomba()
        bomba.explotar()
    """


    def __init__(self, x=0, y=0):
        Animacion.__init__(self, pilas.imagenes.Grilla("bomba.png", 2), ciclica=True, x=x, y=y)
        self.radio_de_colision = 25
        self.aprender(pilas.habilidades.PuedeExplotar)

    def explotar(self):
        # Este metodo solo exite para compativilidad hacia atras.
        self.eliminar()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Animacion

class Moneda(Animacion):

    def __init__(self, x=0, y=0):
        Animacion.__init__(self, pilas.imagenes.Grilla("moneda.png", 8), ciclica=True, x=x, y=y)



import pilas

pilas.iniciar()
mono = pilas.actores.Mono()
mono.escala = 1
mono.rotacion = 0

def girar():
    mono.rotacion = mono.rotacion + 1
    return True

pilas.mundo.agregar_tarea(0, girar)

pilas.avisar("Creando una tarea interminable de girar...")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
import random
import unittest
import pilas

class TestSequenceFunctions(unittest.TestCase):

    def setUp(self):
        pass

    def test_mono_attributes(self):
        mono = pilas.actores.Mono()

        # Verifica que las rotaciones alteren el estado del personaje.
        mono.x = 100
        mono.y = 100

        self.assertEqual(mono.x, 100)
        self.assertEqual(mono.y, 100)

        # Se asegura que inicialmente no tenga rotacion asignada.
        self.assertEqual(mono.rotacion, 0)

        # Verifica que las rotaciones alteren el estado del personaje.
        mono.rotacion = 180
        self.assertEqual(mono.rotacion, 180)

        mono.rotacion = 20
        self.assertEqual(mono.rotacion, 20)

        mono.rotacion = 400
        self.assertEqual(mono.rotacion, (400 % 360))

        # Analiza que el personaje se ha agregado a la lista de actores.
        self.assertTrue(mono in pilas.actores.todos)

        # Utiliza los atributos de escala.
        self.assertEqual(mono.escala, 1)
        mono.escala = 5
        self.assertEqual(mono.escala, 5)

        # Ejecuta mas metodos del mono.
        mono.sonreir()
        mono.gritar()

        # Verifica que el personaje se pueda matar.
        mono.eliminar()
        self.assertFalse(mono in pilas.actores.todos)

    def test_posicion_y_magnitud(self):
        mono = pilas.actores.Mono()
        self.assertEqual(mono.x, 0)
        self.assertEqual(mono.y, 0)

        mono.izquierda = mono.izquierda - 100
        self.assertEqual(mono.x, -100)

        mono.derecha = mono.derecha + 100
        self.assertEqual(mono.x, 0)

        mono.arriba = mono.arriba + 100
        self.assertEquals(mono.y, 100)

        mono.abajo = mono.abajo - 100
        self.assertEquals(mono.y, 0)

    def test_colisiones(self):
        mono = pilas.actores.Mono()
        self.assertTrue(mono.colisiona_con_un_punto(0, 0))
        self.assertFalse(mono.colisiona_con_un_punto(200, 200))

if __name__ == '__main__':
    pilas.iniciar()
    unittest.main()
